var Account = function() {
    this.onLogin = function() {};
    this.onBeforeLogout = function() {};
    this.onLogout = function() {};
    this.$inited = new $.Deferred();
    this.options = {};

    var self = this;
    this.token = null;


    this.init = function() {
        var candidate = localStorage.getItem('auth_token');
        if (candidate) {
            self.token = candidate;
            this.fetch().always(function() {
                self.$inited.resolve();
            });
        } else {
            self.$inited.resolve();
        }
    };

    this.fetch = function() {
        if (!self.token) {
            var $fetched = new $.Deferred();
            $fetched.resolve();
            return $fetched.promise();
        }

        return $.get(schema + '://' + serverUrl + '/account/get-info', {
            app_id: appID,
            token: self.token,
        }).done(function(response) {
            try {
                var result = $.parseJSON(response);
                for (var key in result) {
                    if (result.hasOwnProperty(key)) {
                        self.options[key] = result[key];
                    }
                }
            } catch (e) {
                self.logout();
            }
        }).fail(function() {
            self.logout();
        });
    };

    this.save = function() {
        if (!self.token) {
            return;
        }
        var request = {token: account.token};
        for (var key in self.options) {
            if (self.options.hasOwnProperty(key)) {
                request[key] = self.options[key];
            }
        }
        return $.post(
            schema + '://' + serverUrl + '/account/save-data', request
        );
    };

    this.isAuthorized = function() {
        return !!self.token;
    };

    this.login = function(new_token) {
        self.token = new_token;
        localStorage['auth_token'] = self.token;
        return self.fetch().then(function() {
            self.isAuthorized() && self.onLogin();
        });
    };

    this.logout = function() {
        self.onBeforeLogout();
        localStorage.removeItem('auth_token');
        self.token = null;
        self.nickname = null;
        self.onLogout();
    };
};
