(function() {
    var l10n = {
        'ru-RU': {
            settings: 'Настройка',
            sign_out: 'Выход',
            back: 'Назад',
        },
        'en-US': {
            settings: 'Settings',
            sign_out: 'Sign out',
            back: 'Back',
        }
    };


    /**
     * Класс для работы с возможностями личного кабинета.
     */
    var AccountOptions = function() {
        /**
         * Массив элементов для рендеринга бокового меню / страницы возможностей
         */
        this.options = [];

        /**
         * Добавляет опцию в возможности.
         *
         * @param {object} option   Опция
         */
        this.prependOption = function(option) {
            this.options.unshift(option);
        };

        /**
         * Добавляет опцию в возможности.
         *
         * @param {object} option   Опция
         */
        this.appendOption = function(option) {
            this.options.push(option);
        };

        /**
         * Отрисовывает боковое меню со списком возможностей.
         */
        this.renderSideMenu = function() {
            var $menu = $('.js-left-menu');
            $menu.html('');
            var template = $('#menu-element').text();

            var $backElement = $(template).addClass('js-back-btn');
            $backElement.find('a')
                .removeClass('close-panel').click(restoreSideMenu);
            $backElement.find('.item-title').text(l10n[lang].back);
            $backElement.find('.icon').prop('src', 'img/back.png');
            $menu.append($backElement);

            for (var i = 0; i < this.options.length; i++) {
                var option = this.options[i];
                var $item = $(template);
                var $a = $item.find('a');
                option.keepPanel && $a.removeClass('close-panel');
                $a.click(option.action);
                $item.find('.item-title').text(option.title);
                option.icon && $item.find('.icon').prop('src', option.icon);
                option.class && $item.find('.icon').replaceWith(
                    '<span class="' + option.class + '"></span>'
                );
                $menu.append($item);
            }
        };
    };

    // Экспортирует объект в глобальную область видимости.
    window.accountOptions = new AccountOptions();

    framework7AppReady.then(function() {
        // Экспортирует объект в глобальную область видимости.
        accountOptions.appendOption({
            icon: null,
            class: 'fa fa-2x fa-cog',
            title: l10n[lang].settings,
            keepPanel: false,
            action: function() {
                mainView.router.load({url: 'account.html'});
            },
        });
        accountOptions.appendOption({
            icon: null,
            class: 'fa fa-2x fa-sign-out',
            title: l10n[lang].sign_out,
            keepPanel: false,
            action: function() {
                account.logout();
                restoreSideMenu();
                mainView.router.load({url: 'index.html'});
            },
        });
    });

    /**
     * Восстанавливает боковое меню по умолчанию.
     */
    function restoreSideMenu() {
        renderSideMenu($('.js-left-menu'), mainConfig.instances);
    }
})();
