(function() {
    var l10n = {
        'ru-RU': {
            nickname: 'Ник',
            save: 'Сохранить',
            logout: 'Выйти',
            saving: 'Сохранение',
            save_error: 'Во время сохранения произошла ошибка. Попробуйте позже.',
            save_success: 'Сохранение успешно',
        },
        'en-US': {
            nickname: 'Nickname',
            save: 'Save',
            saving: 'Saving',
            save_error: 'An error occured during saving process. Please, try again later.',
            save_success: 'Successfully saved',
            logout: 'Logout',
        }
    };


    /**
     * Класс для работы с настройками личного кабинета.
     */
    var AccountSettings = function() {
        var that = this;

        this.settings = [];

        /**
         * Добавляет опцию в возможности.
         *
         * @param {object} setting   Настройка
         */
        this.prependSetting = function(setting) {
            this.settings.unshift(setting);
        };

        /**
         * Отрисовывает боковое меню со списком возможностей.
         *
         * @param {jQuery} $selector   Selector to render list to.
         */
        this.renderOptions = function($selector) {
            var objects = {};
            var $form = $('<div class="account-form content-block"></div>');

            for (var i = 0; i < this.settings.length; i++) {
                var setting = this.settings[i];
                var $settingRow = $('<label></label>');
                if (setting.title) {
                    $settingRow.append('<span>' + setting.title + ':</span>');
                }
                var $settingForm = setting.render();
                objects[setting.name] = $settingForm;
                $settingRow.append($settingForm);
                $form.append($settingRow);
            }

            var $saveBtn = $([
                '<a href="#" class="button js-save save">',
                    l10n[lang].save,
                '</a>',
            ].join(''));
            $form.append($saveBtn);

            $selector.append($form);

            $saveBtn.click(function() {
                for (var i = 0; i < that.settings.length; i++) {
                    var setting = that.settings[i];
                    account.options[setting.name] =
                        setting.getValue(objects[setting.name]);
                }
                myApp.showPreloader(l10n[lang].saving);
                account.save()
                    .always(function() {
                        myApp.hidePreloader();
                    }).done(function(response) {
                        if (response !== '') {
                            myApp.alert(l10n[lang].save_error, '');
                        } else {
                            myApp.alert(l10n[lang].save_success, '');
                        }
                    }).fail(function() {
                        myApp.alert(l10n[lang].save_error, '');
                    });
            });
        };
    };

    // Экспортирует объект в глобальную область видимости.
    window.accountSettings = new AccountSettings();

    framework7AppReady.then(function() {
        accountSettings.prependSetting({
            name: 'chat_nickname',
            title: l10n[lang].nickname,
            /**
             * Renders div with required elements, init handlers.
             * This function should use "account" singleton to get its options.
             *
             * @return {jQuery}   Rendered setting.
             */
            render: function() {
                var $input = $('<input type="text" name="chat_nickname" />');
                $input.val(account.options.chat_nickname);
                return $('<div></div>').append($input);
            },
            /**
             * Returns value to store
             *
             * @param {jQuery} $object   Object which was returned by
             *                           "render" method.
             *
             * @return {string}   Option value.
             */
            getValue: function($object) {
                return $object.find('[name="chat_nickname"]').val();
            }
        });
    });
})();
