l10n.forgot_password_form = {
    'ru-RU': {
        enter_registration_email: 'Введите email, указанный при регистрации',
        recover: 'Восстановить',
        back: 'Вернуться',
        choose_email: 'Укажите email',
        recovering_success: 'Пароль успешно отправлен на почту',
        recovering_error: 'Во время восстановления произошла ошибка. Пожалуйста, проверьте написание почты и попробуйте еще раз.',
        sending: 'Отправка...',
    },
    'en-US': {
        enter_registration_email: 'Please, enter email which was used on registration',
        recover: 'Send',
        back: 'Back',
        choose_email: 'Enter email',
        recovering_success: 'Password has been sent to email',
        recovering_error: 'An error occured during password recovering. Please, check email and try again.',
        sending: 'Sending...',
    }
};


var ForgotPasswordForm = function(options) {
    options = Object.assign({
        container: 'body',
        onSentPassword: function() {},
        onBackClick: function() {},
    }, options || {});


    /**
     * Renders login form and initializes handlers.
     */
    this.render = function() {
        var $container = $(options.container);
        $container.empty();

        var $form = $(buildForgotPasswordFormTemplate());

        $form.find('.js-send-password').click(function() {
            var email = $form.find('[name="email"]').val();
            if (!email) {
                myApp.alert(l10n['forgot_password_form'][lang].choose_email, '');
                return;
            }

            myApp.showPreloader(l10n['forgot_password_form'][lang].sending);
            $.post(schema + '://' + serverUrl + '/account/recover-password', {
                app_id: appID,
                email: email,
            }).done(function(response) {
                if (response !== '') {
                    myApp.alert(
                        l10n['forgot_password_form'][lang].recovering_error, ''
                    );
                    return;
                }
                myApp.alert(
                    l10n['forgot_password_form'][lang].recovering_success, '',
                    function() {
                        options.onSentPassword();
                    }
                );
            }).fail(function() {
                myApp.alert(
                    l10n['forgot_password_form'][lang].recovering_error, ''
                );
            }).always(function() {
                myApp.hidePreloader();
            });
        });
        $form.find('.js-back').click(options.onBackClick);
        $container.append($form);
    };
};


/**
 * Returns registration form template.
 *
 * @return {string}   Template string.
 */
function buildForgotPasswordFormTemplate() {
    return [
        '<div class="account-form forgot-password content-block">',
            '<label>',
                '<span>' + l10n['forgot_password_form'][lang].enter_registration_email + ':</span>',
                '<input type="text" placeholder="example@gmail.com"',
                       'name="email" />',
            '</label>',
            '<a href="#" class="button js-send-password">',
                l10n['forgot_password_form'][lang].recover,
            '</a>',
            '<a href="#" class="button js-back">',
                l10n['forgot_password_form'][lang].back,
            '</a>',
        '</div>',
    ].join('');
}
