l10n.login_form = {
    'ru-RU': {
        fill_all_fields: 'Заполните все поля',
        login: 'Войти',
        password: 'Пароль',
        forgot_password: 'Забыли пароль?',
        dont_have_account: 'Нет аккаунта?',
        registration: 'Зарегистрироваться',
        authorize: 'Авторизация...',
        authorize_error: 'Во время авторизации произошла ошибка. Попробуйте позже.',
        account_not_found: 'Неверно введена почта или пароль',
    },
    'en-US': {
        fill_all_fields: 'Please, fill all fields',
        login: 'Login',
        password: 'Password',
        forgot_password: 'Forgot password?',
        dont_have_account: 'Do not have account?',
        registration: 'Registration',
        authorize: 'Authorizing...',
        authorize_error: 'An error occured during authorization. Please, try again later.',
        account_not_found: 'Invalid email or password',
    }
};


/**
 * Describes login form.
 *
 * @param{object} options   Options to initialize form.
 */
var LoginForm = function(options) {
    options = Object.assign({
        container: 'body',
        registration_type: LoginForm.REGISTRATION_FREE,
        onSuccessfulLogin: function() {},
        onRecoverPasswordClick: function() {},
        onRegistrationClick: function() {},
    }, options || {});


    /**
     * Renders login form and initializes handlers.
     */
    this.render = function() {
        var $container = $(options.container);
        $container.empty();

        var $form = $(buildLoginFormTemplate());
        if (options.registration_type === LoginForm.REGISTRATION_CLOSED) {
            $form.find('.register').remove();
        }

        var $registerBtn = $form.find('.js-show-register-form');
        $registerBtn.click(options.onRegistrationClick);
        $container.append($form);

        $form.find('.js-login').click(function() {
            myApp.showPreloader(l10n['login_form'][lang].authorize);
            var $email = $form.find('[name="email"]');
            var $password = $form.find('[name="password"]');

            $.post(schema + '://' + serverUrl + '/account/login', {
                app_id: appID,
                email: $email.val(),
                password: $password.val(),
            }).done(loginHandler)
              .fail(function() {
                myApp.alert(l10n['login_form'][lang].authorize_error, '');
            }).always(function() {
                myApp.hidePreloader();
            });
        });

        $form.find('.js-forgot-password').click(options.onRecoverPasswordClick);


        /**
         * Account login request handler.
         *
         * @param {string} response   Response.
         */
        function loginHandler(response) {
            try {
                var result = $.parseJSON(response);
                if (result.error) {
                    switch (result.error) {
                    case 1:
                    case 2:
                        myApp.alert(l10n['login_form'][lang].fill_all_fields, '');
                        return;
                    case 5:
                        myApp.alert(l10n['login_form'][lang].account_not_found, '');
                        return;
                    default:
                        myApp.alert(
                            l10n['login_form'][lang].authorize_error, ''
                        );
                        return;
                    }
                }
                if (!result.success || !result.token) {
                    myApp.alert(l10n['login_form'][lang].authorize_error, '');
                    return;
                }
                account.login(result.token).then(options.onSuccessfulLogin);
            } catch (e) {
                myApp.alert(l10n['login_form'][lang].authorize_error, '');
            }
        }
    };
};

LoginForm.REGISTRATION_DISABLED = 0;
LoginForm.REGISTRATION_FREE = 1;
LoginForm.REGISTRATION_CLOSED = 2;


/**
 * Returns login form template.
 *
 * @return {string}   Template string.
 */
function buildLoginFormTemplate() {
    return [
        '<form class="account-form login-form content-block">',
            '<input type="text" name="email" ',
                   'placeholder="Email: ivanov@mail.ru"/>',
            '<input type="password" name="password" ',
                   'placeholder="' + l10n['login_form'][lang].password + '"/>',
            '<a href="#" class="js-forgot-password forgot-password">',
                l10n['login_form'][lang].forgot_password,
            '</a>',
            '<div style="clear: both;"></div>',
            '<a href="#" class="button js-login">',
                l10n['login_form'][lang].login,
            '</a>',
            '<div class="register">',
                '<small>',
                    l10n['login_form'][lang].dont_have_account,
                '</small>',
                '<a href="#" class="button js-show-register-form">',
                    l10n['login_form'][lang].registration,
                '</a>',
            '</div>',
        '</form>',
    ].join('');
}
