l10n.registration_form = {
    'ru-RU': {
        password: 'Пароль',
        repeat_password: 'Повторите пароль',
        registration: 'Зарегистрироваться',
        back: 'Вернуться',
        used_email: 'Email уже используется. Укажите другой email',
        choose_email: 'Укажите email',
        choose_password: 'Укажите пароль',
        different_passwords: 'Пароли не совпадают',
        registration_in_process: 'Регистрация...',
        registration_success: 'Регистрация успешна. Теперь вы можете войти под вашими учетными данными.',
        registration_error: 'Во время регистрации произошла ошибка. Попробуйте позже.',
    },
    'en-US': {
        password: 'Password',
        repeat_password: 'Repeat password',
        registration: 'Register',
        back: 'Back',
        used_email: 'Email is already used. Choose another one.',
        choose_email: 'Enter email',
        choose_password: 'Enter password',
        different_passwords: 'Passwords are different',
        registration_in_process: 'Registration...',
        registration_success: 'Registration was successful. Now you can login using your credentials.',
        registration_error: 'An error occured during registration. Please, try again later.',
    }
};


/**
 * Describes registration form.
 *
 * @param {object} options    Options to init form.
 */
var RegistrationForm = function(options) {
    options = Object.assign({
        container: 'body',
        onBack: function() {},
    }, options || {});


    /**
     * Renders registration form and initializes handlers.
     */
    this.render = function() {
        var $container = $(options.container);
        var $form = $(buildRegistrationFormTemplate());

        $container.append($form);
        $form.find('.js-back-to-login').click(options.onBack);

        $form.find('.js-register').click(function() {
            var error;
            if (error = validateForm($form)) {
                myApp.alert(error, '');
                return;
            }
            myApp.showPreloader(
                l10n['registration_form'][lang].registration_in_process
            );

            var $email = $form.find('[name="email"]');
            var $password = $form.find('[name="password"]');

            $.post(schema + '://' + serverUrl + '/account/signup', {
                app_id: appID,
                email: $email.val(),
                password: $password.val(),
            }).done(registrationHandler)
              .fail(function() {
                myApp.alert(
                    l10n['registration_form'][lang].registration_error, ''
                );
            }).always(function() {
                myApp.hidePreloader();
            });
        });


        /**
         * Handles registration response.
         *
         * @param {string} response   Response.
         */
        function registrationHandler(response) {
            try {
                var result = $.parseJSON(response);
                if (result.error) {
                    switch (result.error) {
                    case 1:
                        myApp.alert(
                            l10n['registration_form'][lang].used_email, ''
                        );
                        return;
                    default:
                        myApp.alert(
                            l10n['registration_form'][lang].registration_error,
                            ''
                        );
                        return;
                    }
                }
            } catch (e) {
                myApp.alert(
                    l10n['registration_form'][lang].registration_error, ''
                );
                return;
            }
            myApp.alert(
                l10n['registration_form'][lang].registration_success, ''
            );
            options.onBack();
        }
    };


    /**
     * Validates registration form. Returns error if any.
     *
     * @param {jQuery} $form   Form to validate.
     *
     * @return {null|string}
     */
    function validateForm($form) {
        var $email = $form.find('[name="email"]');
        var $password = $form.find('[name="password"]');
        var $confirmPassword = $form.find('[name="confirm-password"]');

        if (!$email.val()) {
            return l10n['registration_form'][lang].choose_email;
        }
        if (!$password.val() || !$confirmPassword.val()) {
            return l10n['registration_form'][lang].choose_password;
        }
        if ($password.val() !== $confirmPassword.val()) {
            return l10n['registration_form'][lang].different_passwords;
        }
        return null;
    }
};


/**
 * Returns registration form template.
 *
 * @return {string}   Template string.
 */
function buildRegistrationFormTemplate() {
    return [
        '<form class="account-form registration-form content-block">',
            '<input type="text" name="email"',
                   'placeholder="Email: ivanov@mail.ru"/>',
            '<input type="password" name="password" ',
                   'placeholder="' + l10n['registration_form'][lang].password + '"/>',
            '<input type="password" name="confirm-password"',
                   'placeholder="' + l10n['registration_form'][lang].repeat_password + '"/>',
            '<a href="#" class="button js-register">',
                l10n['registration_form'][lang].registration,
            '</a>',
            '<a href="#" class="js-back-to-login button">',
                l10n['registration_form'][lang].back,
            '</a>',
        '</form>',
    ].join('');
}
