/**
 * Класс для отрисовки компонента.
 */
function LinkEditComponent() {
    /**
     * "Отрисовка" компонента.
     *
     * @param {Object} selector   Объект-контейнер, куда будут добавляться
     *                            элементы.
     * @param {string} config     Конфиг для заполнения компонента в формате
     *                            JSON.
     */
    this.renderComponent = function(selector, config) {
        var data = $.parseJSON(config),
            loader = new ExternalLinkLoader({
                link: data.text,
                container: selector,
                onInit: function() {
                    myApp.showPreloader(l10n.common[lang].loading);
                    selector.append(
                        $('<span>')
                            .addClass(
                                'preloader custom-preloader ' +
                                'js-external-link-preloader'
                            )
                            .css({
                                width: '42px',
                                height: '42px',
                                marginTop: '45vh',
                                marginLeft: '45vw',
                            })
                    );
                },
                onInitLoaded: function(innerLoader) {
                    myApp.hidePreloader();
                    myApp.closePanel(false);
                    $(document)
                        .on('panel:open.webLink', innerLoader.hideLink)
                        .on('panel:closed.webLink', innerLoader.showLink);
                },
                onError: myApp.hidePreloader,
                onShow: function() {
                    $('.js-external-link-preloader').hide();
                },
                onHide: function() {
                    $('.js-external-link-preloader').show();
                },
            });

        $(document)
            .on('page:back.webLink', onMoveToAnotherPage)
            .on('page:beforeanimation.webLink', onMoveToAnotherPage);

        /**
         * Reset all callbacks and hides view.
         */
        function onMoveToAnotherPage() {
            $(document)
                .off('panel:open.webLink')
                .off('panel:closed.webLink')
                .off('page:back.webLink')
                .off('page:beforeanimation.webLink');
            loader.hideLink();
        }
    };
}
