/**
 * Класс для отрисовки компонента
 */
function NewsComponent() {
    var self = this;
    var isShowingDetailedNew = false;
    var curInstanceId;
    var curRssUrl;
    var $contentDiv;
    var $newsDiv;
    var $emptyDiv;
    var detailCard;


    /**
     * "Отрисовка" компонента.
     *
     * @param {object} selector  Контейнер для рендеринга компонента.
     * @param {string} config    Конфиг инстанса.
     * @param {int} instanceId   ID инстанса компонента.
     * @param {null|int} newId   ID новости для начала просмотра.
     */
    this.renderComponent = function(selector, config, instanceId, newId) {
        $(document).off('backbutton')
                   .on('backbutton', onBackKeyDown);

        var data = $.parseJSON(config);

        curInstanceId = instanceId;
        curRssUrl = data.rss_url;
        this.rss = curRssUrl;
        $contentDiv = $('<div class="page-content" style="display: none"></div>');
        $newsDiv = $('<div class="list-block media-list"><ul></ul></div>');
        $emptyDiv = $('<div class="news-list" style="display: none">' +
            '<div style="padding: 25px" class="">' +
            '{0}'.format(l10n.news[lang].news_empty) +
            '</div>' +
            '</div>');

        $contentDiv.append($newsDiv);
        $contentDiv.append($emptyDiv);
        selector.append($contentDiv);

        detailCard = new DetailNewsCard();
        detailCard.init(this);
        selector.append(detailCard.visualElement);

        this.getNewsContent(curRssUrl).then(function() {
            if (newId) {
                $('.js-news-item[data-id="' + newId + '"]').trigger('click');
            }
        });
    };

    /**
     * Обработчик нажатия кнопки назад на android-телефонах.
     */
    function onBackKeyDown() {
        if (isShowingDetailedNew) {
            showMainContent();
            self.getNewsContent(curRssUrl);
        } else {
            mainView.router.load({url: 'index.html'});
        }
    }

    /**
     * Запрос сервера содержимого
     *
     * @param {string} rssUrl - урла rss, если ручной ввод - пустая
     *
     * @return {promise}   Promise of getting content.
     */
    this.getNewsContent = function(rssUrl) {
        var request = {
            instance_id: curInstanceId,
            rss_url: rssUrl,
        };
        if ((typeof(geocoder) !== 'undefined') && geocoder.city) {
            Object.assign(request, {city: geocoder.city});
        }

        return $.get(
            schema + '://' + serverUrl + '/news/get-news-content',
            request, refreshData
        ).fail(function(xhr, status) {
            myApp.alert(status);
        });
    };

    /**
     * Отображение содержимого RSS-ленты.
     *
     * @param {string} data - данные в JSON формате.
     */
    function refreshData(data) {
        var newsList = $.parseJSON(data);
        var $ul = $newsDiv.find('ul');
        $ul.empty();
        $emptyDiv.hide();

        var now = new Date();
        newsList = newsList.filter(function(item) {
            if (item.start_date && (now < new Date(item.start_date))) {
                return false;
            }
            if (item.end_date && (now > new Date(item.end_date))) {
                return false;
            }
            return true;
        });

        for (var i = 0; i < newsList.length; i++) {
            var news = newsList[i];
            var $li = $(
                '<li>' +
                    '<a class="item-link item-content js-news-item" ' +
                       'data-id="' + news.id + '" href="#"></a>' +
                '</li>'
            );
            var $a = $li.find('a');
            if (news.image) {
                var $imgContent = $('<div class="item-media"><img src="{0}" width="120"></div>'.format(news.image));
                $a.append($imgContent);
            }
            var $divContent = $('<div class="item-inner"><div class="article-title">{0}</div><div class="article-date">{1}</div></div>'.format(news.name, news.date));
            $a.append($divContent);
            $a.click({news: news}, getOneNews);
            $ul.append($li);
        }

        if(newsList.length === 0) {
            $emptyDiv.show();
        }

        showMainContent();
    }

    /**
     * Перейти на новость
     *
     * @param {object} event - event объект события.
     */
    function getOneNews(event) {
        event.stopPropagation();
        showDetailCard(event.data.news);
    }

    /**
     * Показать подробную карточку новости.
     *
     * @param {object} news - объект новости.
     */
    function showDetailCard(news) {
        showDetailedNew();
        detailCard.createDetailNewsCard(news);
    }

    /**
     * Показывает блок основного контента.
     */
    function showMainContent() {
        $contentDiv.css('display', 'block');
        detailCard.visualElement.css('display', 'none');
        isShowingDetailedNew = false;
    }

    /**
     * Показывает блок детальной новости.
     */
    function showDetailedNew() {
        $contentDiv.css('display', 'none');
        detailCard.visualElement.css('display', 'block');
        isShowingDetailedNew = true;
    }
}

/**
 * Класс для работы с детальной карточкой новости
 */
function DetailNewsCard() {
    var self = this;
    var parent = null;
    this.visualElement = null;

    /**
     * Инициализация детали.
     * @param {object} newsList - список новостей.
     */
    this.init = function(newsList) {
        createVisualElement();
        parent = newsList;
    };

    /**
     * Создание основного визуального элемента детальной карточки
     */
    function createVisualElement() {
        var template = '<div class="page-content" style="display: none"></div>';
        self.visualElement = $(template);
    }

    /**
     * Создать HTML представление подробной карточки новости.
     *
     * @param {object} news - данные новости.
     */
    this.createDetailNewsCard = function(news) {
        var origLink = '';
        if(news.link !== undefined) {
            origLink =
                '<div class="card-content-inner">' +
                    '<a class="external item-link" target="_blank" rel="noopener noreferrer"  href="' +
                        '{0}'.format(news.link) +
                        '">' +
                        l10n.news[lang].news_link +
                    '</a>' +
                '</div>';
        }

        var template =
            '<div class="offer">' +
                '<div class="card">' +
                    '<div class="card-content">' +
                        '<div class="card-content-inner">' +
                            '<div class="offer-item-img"></div>' +
                        '</div>' +
                        '<div id="js-component-news" class="card-content-inner wrap">' +
                            '<p>{0}<br></p>'.format(news.text) +
                        '</div>' +
                         origLink +
                    '</div>' +
                '</div>' +
                '<div class="back_button_wrapper" style="padding: 0 20px;">' +
                    '<a class="button button-outline" style="width: auto;" ' +
                       'id="js-back-btn" href="#">' +
                        l10n.news[lang].back +
                    '</a>' +
                '</div>' +
            '</div>';

        var $card = $(template);
        // Класс external необходим, иначе ссылки открываются
        // через Ajax, что приводит к ошибке.
        $card.find('#js-component-news').find('a').addClass('external');
        setDetailNewsImage($card, news);

        var $a = $card.find('#js-back-btn');
        $a.click(function() {
            parent.getNewsContent(parent.rss || '');
        });

        self.visualElement.empty();
        self.visualElement.append($card);
    };

    /**
     * Установить изображение
     *
     * @param {object} $card - jQuery объект - карточка новости.
     * @param {object} news - новость.
     */
    function setDetailNewsImage($card, news) {
        var images = [];

        if(news.image !== undefined && news.image !== null && news.image !== '') {
            images.push(news.image);
        }

        if(images.length > 0) {
            var $imageDiv = $card.find('.offer-item-img');
            $imageDiv.css('background-image', 'url("{0}")'.format(images[0]));
        }
    }
}
