/**
 * Массив в который сохраняется структура каталогов магазина, по мере перехода
 * между каталогами
 */
var leftMenu = [];
var favoritesColor = 'blue';

/**
 * Класс для отрисовки компонента
 */
function StoreComponent() {
    var curInstanceId;
    var $contentDiv;
    var $categoriesDiv;
    var $goodsDiv;
    var $emptyDiv;
    var $notFoundDiv;
    var searchBar;
    var shopCart;
    var detailCard;
    var shopOrder;

    var historyIndex;
    var history;
    var navigationAction;
    var componentConfig;
    var favorites;

    var basketHistory = 'basket';
    var orderHistory = 'order';
    var productHistory = 'product:';
    var searchHistory = 'search:';

    /**
     * "Отрисовка" компонента.
     *
     * @param {object} selector - Объект-контейнер, куда будут добавляться элементы.
     * @param {string} config - Конфиг для заполнения формы в формате JSON.
     * @param {int} instanceId - id инстанса компонента.
     * @param {int} categoryId - id категории с которой нужно начать просмотр каталога.
     * @param {boolean} startWithCart - открыть ли сразу корзину.
     * @param {int} productId - id товара. Если есть, то сразу он откроется.
     * @param {string} searchQuery - текст для поиска.
     */
    this.renderComponent = function(selector, config, instanceId, categoryId, startWithCart, productId, searchQuery) {
        $(document).off('backbutton');
        $(document).on('backbutton', onBackKeyDown);

        componentConfig = $.parseJSON(config);

        favorites = $.parseJSON(localStorage.getItem(globalDeviceId + '---' + instanceId));

        if(!Array.isArray(favorites)) {
            favorites = [];
        }

        if (componentConfig.payments === undefined || componentConfig.payments === null) {
            componentConfig.payments = [{
                id: 'cash'
            }];
        }

        history = [];
        historyIndex = -1;

        curInstanceId = instanceId;

        $contentDiv = $('<div class="page-content" style="display: none">' +
                            '<form class="searchbar searchbar-init" style="background-color: ' + mainConfig.theme.catalog_search_background + '">' +
                                '<div class="searchbar-input">' +
                                    '<input type="search" placeholder="{0}">'.format(l10n.store[lang].catalog_search) +
                                    '<a href="#" class="searchbar-clear"></a>'+
                                '</div>'+
                            '</form>'+
                        '</div>');
        $categoriesDiv = $('<div class="list-block"><ul></ul></div>');
        $goodsDiv = $('<div class="category-list"></div>');
        $emptyDiv = $('<div class="category-list" style="display: none">' +
                            '<div style="padding: 25px" class="">' +
                                l10n.store[lang].goods_empty +
                            '</div>' +
                      '</div>');

        $notFoundDiv = $('<div class="category-list" style="display: none">' +
                            '<div style="padding: 25px" class="">' +
                                l10n.store[lang].no_search_results +
                            '</div>' +
                        '</div>');

        $contentDiv.append($categoriesDiv);
        $contentDiv.append($goodsDiv);
        $contentDiv.append($notFoundDiv);
        selector.append($contentDiv);

        detailCard = new DetailProductCard();
        detailCard.init(componentConfig, addToCart, removeFromCart, favoritesClick, favorites);
        selector.append(detailCard.visualElement);

        shopCart = new ShopCart();
        shopCart.init(instanceId, globalDeviceId, componentConfig, getProduct, showOrder);
        selector.append(shopCart.visualElement);

        shopOrder = new ShopOrder();
        shopOrder.init(instanceId, globalDeviceId, componentConfig,
            function() {
                history = [];
                historyIndex = -1;
            });
        selector.append(shopOrder.visualElement);

        searchBar = myApp.searchbar($contentDiv.find('.searchbar'));
        searchBar.input.on('keyup', function(e) {
            if (e.keyCode == 13 && searchBar.input.val() !== '') {
                search(searchBar.input.val());
            }
        });

        if(!componentConfig.settings.showSearch) {
            $contentDiv.find('.searchbar').hide();
        }

        var obj = {data: {categoryId: categoryId}};

        if(startWithCart) {
            showCart();
        } else if(searchQuery !== undefined && searchQuery !== null && searchQuery !== '') {
            search(searchQuery);
        } else if(productId !== undefined && productId !== null) {
            getProduct({data: {productId: productId}});
        } else {
            getCategoryContent(obj);
        }
    };

    /**
     * Обработчик нажатия кнопки назад на android-телефонах.
     */
    function onBackKeyDown() {
        if(historyIndex < 1) {
            mainView.router.load({url: 'index.html'});
            return;
        }

        navigationAction = 'back';
        var historyItem = history[historyIndex-1];
        var strHistoryItem = historyItem.toString();

        var currentHistoryItem = history[historyIndex];

        if(currentHistoryItem === orderHistory) {
            if(shopOrder.back()) {
                return;
            }
        }

        if(historyItem === basketHistory) {
            showCart();
        } else if(strHistoryItem.startsWith(productHistory)) {
            var obj = {data: {productId: parseInt(strHistoryItem.replace(productHistory, ''))}};
            getProduct(obj);
        } else if(strHistoryItem.startsWith(searchHistory)) {
            search(strHistoryItem.replace(searchHistory, ''));
        } else {
            var obj = {data: {categoryId: historyItem}};
            getCategoryContent(obj);
        }
    }

    /**
     * Запрос сервера содержимого категории
     *
     * @param {object} event - event объект события
     */
    function getCategoryContent(event) {
        var options = {
            instance_id: curInstanceId,
            device_id: globalDeviceId,
            category_id: event.data.categoryId
        };
        if ((typeof(geocoder) !== 'undefined') && geocoder.city) {
            options.city = geocoder.city;
        }

        $.get(
            schema + '://' + serverUrl + '/shop/get-category-content', options,
            function(data) {
                refreshData(data, event.data.categoryId);
            }
        ).fail(function(xhr, status, error) {
            resetNavigationAction();
            myApp.alert(status);
        });
    }

    /**
     * Отображение содержимого текущей категории.
     *
     * @param {string} data - данные в JSON формате.
     * @param {int} categoryId - id категории.
     */
    function refreshData(data, categoryId) {
        updateHistory(categoryId);

        var catalog = $.parseJSON(data);

        var $ul = $categoriesDiv.find('ul');
        $ul.empty();
        $goodsDiv.empty();
        $emptyDiv.hide();
        $notFoundDiv.hide();

        // Filter by not fit places.
        if ((typeof(geocoder) !== 'undefined') && geocoder.city) {
            if (catalog.categories.length) {
                catalog.categories =
                    filterItemsByCity(catalog.categories, geocoder.city);
            }
            if (catalog.products.length) {
                catalog.products =
                    filterItemsByCity(catalog.products, geocoder.city);
            }
        }

        for (var i = 0; i < catalog.categories.length; i++) {
            var imgTemplate = componentConfig.settings.showCategoryPhoto
                ? '<div class="item-media">' +
                    '<div class="category-item-img-div">' +
                        '<div class="category-item-img"/>' +
                    '</div>' +
                  '</div>'
                : '';

            var category = catalog.categories[i];
            var $li = $('<li><a class="item-link item-content" href="#"></a></li>');
            var $divContent = $(imgTemplate +
                                '<div class="item-inner">' +
                                    '<div class="item-title">{0}</div>'.format(category.name) +
                                    '<div class="item-after">{0}</div>'.format(category.description) +
                                '</div>');

            if(componentConfig.settings.showCategoryPhoto && category.image) {
                var $imageDiv = $divContent.find('.category-item-img');
                $imageDiv.css('background-image', 'url("{0}")'.format(category.image));
            }

            if(!componentConfig.settings.showCategoryDescription) {
                $divContent.find('.item-after').hide();
            }

            var $a = $li.find('a');
            $a.append($divContent);
            $a.click({categoryId: category.id}, getCategoryContent);
            $ul.append($li);
        }

        for (var i = 0; i < catalog.products.length; i++) {
            var product = catalog.products[i];
            $goodsDiv.append(createProductCard(product));
        }

        if(catalog.categories.length === 0 && catalog.products.length === 0) {
            $emptyDiv.show();
        }

        hideDetailCard();
        hideCart();
        hideOrder();
        $contentDiv.css('display', 'block');


        /**
         * Filters given set of items containing places by given city.
         *
         * @param {array} items   Items to filter.
         * @param {string} city   City to filter with.
         *
         * @return {array}  Filtered items.
         */
        function filterItemsByCity(items, city) {
            return items.filter(function(item) {
                if ((item.places === '[]') || (item.places === '')) {
                    return true;
                }
                try {
                    var categories = JSON.parse(item.places);
                    return categories.indexOf(city) !== -1;
                } catch (e) {
                    return false;
                }
            });
        }
    }

    /**
     * Отображение результатов поиска
     *
     * @param {string} data - данные в JSON формате.
     * @param {string} text - текст по которому производился поиск.
     */
    function refreshSearchData(data, text) {
        updateHistory(searchHistory + text);
        searchBar.clear();
        searchBar.disable();

        var products = $.parseJSON(data);

        var $ul = $categoriesDiv.find('ul');
        $ul.empty();
        $goodsDiv.empty();
        $emptyDiv.hide();
        $notFoundDiv.hide();

        for (var i = 0; i < products.length; i++) {
            var product = products[i];
            $goodsDiv.append(createProductCard(product));
        }

        if(products.length === 0) {
            $notFoundDiv.show();
        }

        hideDetailCard();
        hideCart();
        hideOrder();
        $contentDiv.css('display', 'block');
    }

    /**
     * Поиск по каталогу
     *
     * @param {string} text - текст поиска.
     */
    function search(text) {
        $.get(schema + '://' + serverUrl + '/shop/search-products',
            {
                instance_id: curInstanceId,
                device_id: globalDeviceId,
                query: text
            },
            function(data) {
                refreshSearchData(data, text);
            })
            .fail(function(xhr, status, error) {
                resetNavigationAction();
                myApp.alert(status);
            });
    }

    /**
     * Перейти на конкретный товар
     *
     * @param {object} event - event объект события.
     *
     * @return {object} Promise to render card.
     */
    function getProduct(event) {
        if (event.stopPropagation !== undefined) {
            event.stopPropagation();
        }
        return $.get(schema +'://' + serverUrl + '/shop/get-products', {
            instance_id: curInstanceId,
            device_id: globalDeviceId,
            product_ids: [event.data.productId],
        }, function(data) {
            var prods = $.parseJSON(data);
            if (prods.length > 0) {
                showDetailCard(prods[0]);
            }
        })
        .fail(function(xhr, status) {
            resetNavigationAction();
            myApp.alert(status);
        });
    }

    /**
     * Создать HTML представление карточки товара.
     *
     * @param {object} product - данные товара.
     * @return {object} jQuery объект - карточка товара.
     */
    function createProductCard(product) {
        var shareTitle = componentConfig.settings.shareText
            ? componentConfig.settings.shareText
            : l10n.store[lang].share;


        var template =
            '<div class="card offer">' +
                '<div class="card-content">' +
                    '<div class="card-content-inner">' +
                        '<div class="row no-gutter">' +
                            '<div class="col-50">' +
                                '<div class="catalog-item-img-div">' +
                                (product.old_price
                                    ? '<div class="catalog-ribbon">' +
                                          '<span>' +
                                              l10n.store[lang].sale +
                                          '</span>' +
                                      '</div>'
                                    : '') +
                                    '<div class="catalog-item-img"></div>' +
                                '</div>' +
                            '</div>' +
                            '<div class="col-50" id="prodInfo">' +
                                '<div class="catalog_item">{0}</div>'.format(product.name) +
                                '<div class="catalog_price">' +
                                    '<span id="prdPrice">' +
                                        ' {0} {1}'.format(product.price, getLocalizedCurrency(componentConfig.storeCurrency)) +
                                    '</span>' +
                                '</div>' +
                                '<a href="#" id="btnCart" class="button ui_button_text ui_button_background">' +
                                    '{0}'.format(l10n.store[lang].add_to_cart) +
                                '</a>' +

                                '<div class="cart-item-price" style="display: none">' +
                                    '<div class="buttons-row ui_button_background" style="border:1px solid #ccc;border-radius:5px">' +
                                        '<a href="#" id="btnMinus" class="button">-</a>' +
                                        '<span class="button" id="qtycart" style="border-left:1px solid #ccc;border-right:1px solid #ccc">' +
                                            '0' +
                                        '</span>' +
                                        '<a href="#" id="btnPlus" class="button">+</a>' +
                                    '</div>' +
                                '</div>' +
                            '</div>' +
                        '</div>' +
                    '</div>' +
                '</div>' +
                '<div class="card-footer catalog-footer-links">' +
                    '<a href="#" id="shareBtn" class="link ui_text_color">' +
                        '<i class="fa fa-share-alt" style="padding:0 5px;font-size:18px;"></i>' +
                        shareTitle +
                    '</a>' +
                    '<a href="#" class="link ui_text_color favourites">' +
                        '<i class="fa fa-heart"></i>' +
                        '{0}'.format(l10n.store[lang].to_favorites) +
                    '</a>' +
                '</div>' +
            '</div>';

        var $card = $(template);
        setImage($card, product);

        $card.find('.catalog-item-img-div').click({productId: product.id}, getProduct);
        $card.find('#btnCart').click({product: product}, addToCart);
        $card.find('#btnMinus').click({product: product}, removeFromCart);
        $card.find('#btnPlus').click({product: product}, addToCart);

        if(!componentConfig.settings.showSocial) {
            $card.find('#shareBtn').hide();
        }

        if(!componentConfig.settings.showPrices) {
            $card.find('#prdPrice').hide();
        }

        if(!componentConfig.settings.showCart) {
            $card.find('#btnCart').hide();
        }

        var favoritesLink = $card.find('.favourites');
        var shareLink = $card.find('#shareBtn');

        if($.inArray(product.id, favorites) > -1) {
            favoritesLink.css('color', favoritesColor);
        }

        favoritesLink.click({productId: product.id}, favoritesClick);
        shareLink.click(function() {
            copyStringToClipboard(
                schema + '://' + serverUrl +
                '/shop/show-product?product_id=' + product.id
            );
            myApp.alert(l10n.store[lang].link_copied);
        });

        if(product.basket_count > 0) {
            $card.find('#btnCart').css('display', 'none');
            $card.find('.cart-item-price').css('display', 'block');
            $card.find('#qtycart').text(product.basket_count);
        }

        var shortDescription = reduceDescription(product.description);
        if (product.show_short_description && shortDescription
            && (shortDescription !== '...')
        ) {
            $card.find('.card-content-inner').append(
                $('<span></span>').text(shortDescription)
            );
        }

        $(document).off('click.related-product').on(
            'click.related-product', '.related-product', function(event) {
                event.stopPropagation();
                event.preventDefault();
                getProduct({data: {productId: $(this).data('product_id')}})
                    .done(function() {
                        $('.page-content').scrollTop(0, 0);
                    });
            }
        );
        return $card;


        /**
         * Reduces given description to 70 symbols. Adds "..." if it was cut.
         *
         * @param {string} description   Description to cut.
         *
         * @return {string}  Cut description.
         */
        function reduceDescription(description) {
            var stripped = description.replace(/(<([^>]+)>)/ig, '');
            stripped = stripped.substring(0, 70);
            var words = stripped.split(' ');
            words.pop();
            var final = words.join(' ');
            if (final !== stripped) {
                final += '...';
            }
            return final;
        }
    }

    /**
     * Положить избранное в локальное хранилище
     */
    function setFavoritesToStorage() {
        localStorage.setItem(globalDeviceId + '---' + curInstanceId, JSON.stringify(favorites));
    }


    /**
     * Обработчик клика для кнопки Избранное
     *
     * @param {object} event - event объект события.
     */
    function favoritesClick(event) {
        var index = $.inArray(event.data.productId, favorites);

        if(index > -1) {
            $(this).css('color', '');
            favorites.splice(index, 1);
        } else {
            $(this).css('color', favoritesColor);
            favorites.push(event.data.productId);
        }

        setFavoritesToStorage();
    }

    /**
     * Спрятать подробную карточку товара.
     */
    function hideMainContent() {
        $contentDiv.css('display', 'none');
    }

    /**
     * Показать подробную карточку товара.
     *
     * @param {object} product - объект товара.
     */
    function showDetailCard(product) {
        updateHistory(productHistory + product.id);
        hideMainContent();
        hideCart();
        hideOrder();
        detailCard.createDetailProductCard(product);
        detailCard.visualElement.css('display', 'block');
        detailCard.initSwiper();
    }

    /**
     * Спрятать подробную карточку товара.
     */
    function hideDetailCard() {
        detailCard.visualElement.css('display', 'none');
    }

    /**
     * Добавляет товар в корзину.
     *
     * @param {object} event - event объект события.
     */
    function addToCart(event) {
        event.stopPropagation();

        var count = event.data.product.basket_count + 1;

        var callback = function() {
            var $parent = $(event.currentTarget).parent();
            event.data.product.basket_count += 1;
            $parent.find('#qtycart').text(event.data.product.basket_count);
            $parent.find('.cart-item-price').css('display', 'block');
            $parent.find('#btnCart').css('display', 'none');
        };

        shopCart.updateCart(event.data.product.id, count, callback);
    }


    /**
     * Добавляет товар в корзину.
     *
     * @param {object} event - event объект события.
     */
    function removeFromCart(event) {
        event.stopPropagation();

        var count = event.data.product.basket_count - 1;

        var callback = function() {
            var $parent = $(event.currentTarget).parents('#prodInfo');
            event.data.product.basket_count -= 1;
            $parent.find('#qtycart').text(event.data.product.basket_count);

            if(event.data.product.basket_count === 0) {
                $parent.find('.cart-item-price').css('display', 'none');
                $parent.find('#btnCart').css('display', 'block');
            }
        };

        shopCart.updateCart(event.data.product.id, count, callback);
    }

    /**
     * Показать корзину.
     */
    function showCart() {
        shopCart.refreshCart();
        hideDetailCard();
        hideMainContent();
        hideOrder();
        shopCart.visualElement.css('display', 'block');
        updateHistory(basketHistory);
    }

    /**
     * Показать заказ.
     *
     * @param {int} count - кол-во товаров.
     * @param {int} sum - стоимость всех товаров.
     */
    function showOrder(count, sum) {
        shopOrder.refreshOrder();
        hideDetailCard();
        hideMainContent();
        hideCart();
        shopOrder.setFooter(count, sum);
        shopOrder.visualElement.css('display', 'block');
        updateHistory(orderHistory);
    }

    /**
     * Спрятать корзину.
     */
    function hideCart() {
        shopCart.visualElement.css('display', 'none');
    }

    /**
     * Спрятать заказ
     */
    function hideOrder() {
        shopOrder.visualElement.css('display', 'none');
    }

    /**
     * Добавить элемент в историю или удалить последний.
     * @param {object} historyItem - элемент, обозначающий положение для кнопки назад
     */
    function updateHistory(historyItem) {
        if(navigationAction === 'back') {
            historyIndex--;
            history.pop();
        } else{
            history.push(historyItem);
            historyIndex++;
        }

        navigationAction = null;
    }

    /**
     * Сбросить действие при навигации
     */
    function resetNavigationAction() {
        navigationAction = null;
    }
}

/**
 * Функция, выполняющаяся при нажатии на строку меню компонента магазин
 * отправляет запрос на получение списка дочерних каталогов
 *
 * @param {object} params - instance_id - id инстанса, для запроса каталогов
 * id - id каталога, чьи дочерние каталоги мы запрашиваем,
 * если id=0 - запрашивается корневой каталог
 */
function openStore(params) {
    if (leftMenu.length === 0) {
        leftMenu.push({
            category_id: 0,
            category_name: shopInstance.text,
            parent_id: -1
        });
    }

    var options = {
        instance_id: params.instance_id,
        device_id: globalDeviceId,
        category_id: leftMenu[params.current_id].category_id,
        mode: params.mode
    };
    if ((typeof(geocoder) !== 'undefined') && geocoder.city) {
        options.city = geocoder.city;
    }

    $.get(
        schema + '://' + serverUrl + '/shop/get-category-content',
        options, refreshMenu
    ).fail(function(xhr, status, error) {
        myApp.alert(status);
    });

    /**
     * Функция обрабатывает полученный список каталогов, изменяя в соответствии
     * с ним пункты меню на панели слева
     *
     * @param {object} data - JSON объект, представляющий список каталогов
     */
    function refreshMenu(data) {
        var catalog = $.parseJSON(data);
        if (catalog.length > 0) {
            var buf;
            var category;
            $selector = $('.js-left-menu');
            $selector.html('');

            var $menuElement = $($('#menu-element').text());
            var $a = $menuElement.find('a');
            $a.removeClass('close-panel');
            if (leftMenu[params.current_id].parent_id === -1) {
                appendCall($a);
            } else {
                appendCallback( $a, 'openStore',
                    {
                        instance_id: params.instance_id,
                        header_name: leftMenu[params.current_id].category_name,
                        current_id: leftMenu[params.current_id].parent_id,
                        mode: params.mode
                    });
            }
            $menuElement.find('.item-title').text(leftMenu[params.current_id].category_name);
            $menuElement.find('.icon').prop('src', 'img/back.png');
            $selector.append($menuElement);

            for (var i = 0; i < catalog.length; i++) {
                category = catalog[i];
                $menuElement = $($('#menu-element').text());
                $a = $menuElement.find('a');
                $menuElement.find('.item-title').text(category.name);
                if (mainConfig.showCategoryPhoto) {
                    $menuElement.find('.icon').prop('src', category.image);
                } else {
                    $menuElement.find('.icon').hide();
                }
                if (category.has_children) {
                    $a.removeClass('close-panel');
                    buf = getleftMenuId(category.id);
                    if (buf === -1) {
                        leftMenu.push({
                            category_id: category.id,
                            category_name: category.name,
                            parent_id: params.current_id
                        });
                        buf = leftMenu.length - 1;
                    }
                    appendCallback($menuElement.find('a'), 'openStore',
                        {
                            instance_id: params.instance_id,
                            header_name: leftMenu[params.current_id].category_name,
                            current_id: buf,
                            mode: params.mode
                        });
                } else {
                    $a.prop('href', shopInstance.href +
                        '?category_id=' + category.id);
                    $a.attr('data-context', '{"Title": "' + category.name +
                        '", "IsBasketVisible": true}');
                }
                $selector.append($menuElement);
            }
        }
    }

    /**
     * Функция добавляет колбэк к пункту меню. В колбэке снова вызывается
     * функция получения дочерних каталогов, тем самым дерево каталогов
     * продолжает строиться вниз
     *
     * @param {object} $a - jquery объект представляющий ссылку (пункт меню)
     * на которую вешается колбэк
     * @param {string} instance - имя процедуры вызываемой в качестве колбэка
     * @param {object} someParams - параметры процедуры вызываемой в качестве колбэка
     */
    function appendCallback($a, instance, someParams) {
        $a.on('click', function() {
            window[instance].call(window, someParams);
        });
    }

    /**
     * Функция добавляет колбэк к пункту меню. В колбэке вызывается
     * функция построения меню по инстансам (стандартный вид меню)
     *
     * @param {object} $a - jquery объект представляющий ссылку (пункт меню)
     * на которую вешается колбэк
     */
    function appendCall($a) {
        $a.on('click', function() {
            renderSideMenu($('.js-left-menu'), mainConfig.instances);
        });
    }

    /**
     * Функция получает объект из массива leftMenu по его ключевому параметру
     *
     * @param  {number} categoryId - ID категории
     * @return {number} - номер элемента массива
     */
    function getleftMenuId(categoryId) {
        for (var i = 0; i < leftMenu.length; i++) {
            if (leftMenu[i].category_id === categoryId) {
                return i;
            }
        }
        return -1;
    }
}

/**
 * Установить изображение товара
 *
 * @param {object} $card - jQuery объект - карточка товара.
 * @param {object} product - данные товара.
 */
function setImage($card, product) {
    var images = [];

    if(product.image !== undefined && product.image !== null && product.image !== '') {
        images = $.parseJSON(product.image);
    }

    if(images.length > 0) {
        var $imageDiv = $card.find('.catalog-item-img');

        if($imageDiv.length === 0) {
            $imageDiv = $card.find('.offer-item-img');
        }

        $imageDiv.css('background-image', 'url("{0}")'.format(images[0]));
    }
}

/**
 * Взять локализованую валюту
 *
 * @param {string} currencyIso - название валюты в ISO стандарте.
 * @return {string} локализованное название валюты.
 */
function getLocalizedCurrency(currencyIso) {
    var currency = l10n.store[lang][currencyIso];

    if(currency === undefined) {
        return currencyIso;
    }

    return currency;
}

/**
 * Класс для работы с детальной карточкой товара
 */
function DetailProductCard() {
    var self = this;
    var config;
    var addToCartFunc;
    var favoritesClickFunc;
    var removeCardFromCard;
    var favorites;
    var needSwiper;
    this.visualElement = null;

    /**
     * Инициализация корзины.
     * @param {object} storeConfig - конфиг магазина
     * @param {function} addToCart - делегат, метод для добавления в корзину
     * @param {function} removeFromCart - делегат, метод для удаления из корзины
     * @param {function} favoritesClick - делегат, метод для клика "Избранное"
     * @param {array} storeFavorites - массив id товаров.
     */
    this.init = function(storeConfig, addToCart, removeFromCart, favoritesClick, storeFavorites) {
        config = storeConfig;
        addToCartFunc = addToCart;
        removeCardFromCard = removeFromCart;
        favoritesClickFunc = favoritesClick;
        favorites = storeFavorites;
        createVisualElement();
    };

    /**
     * Создание основного визуального элемента детальной карточки
     */
    function createVisualElement() {
        var template = '<div class="page-content" style="display: none"></div>';
        self.visualElement = $(template);
    }

    /**
     * Создать HTML представление подробной карточки товара.
     *
     * @param {object} product - данные товара.
     */
    this.createDetailProductCard = function(product) {
        var charsTitle = config.settings.modificationsTitle
            ? config.settings.modificationsTitle
            : l10n.store[lang].characteristics;

        var description = config.settings.aboutProductTitle
            ? config.settings.aboutProductTitle
            : l10n.store[lang].description;

        var shareTitle = config.settings.shareText
            ? config.settings.shareText
            : l10n.store[lang].share;


        var template =
            '<div class="offer">' +
                '<div class="card">' +
                    '<div class="card-content">' +
                        '<div class="card-content-inner">' +
                            '<div class="offer-item-img">' +
                                '<div class="swiper-container" style="height: 250px">' +
                                    '<div class="swiper-wrapper"></div>' +
                                    '<div class="swiper-pagination"></div>' +
                                '</div>' +
                            '</div>' +
                        '</div>' +
                        '<div class="card-footer">{0}</div>'.format(product.name) +
                    '</div>' +
                '</div>' +
                '<div class="infinite-scroll infinite-scroll-top">' +
                    '<div class="navbar ui_price_layout_background">' +
                        '<div class="navbar-inner">' +
                             '<div id="prdPrice" class="left">' +
                                '<div class="total ui_text_color_secondary">' +
                                    '<div class="total-price">' +
                                        '{0} {1}'.format(product.price, getLocalizedCurrency(config.storeCurrency)) +
                                        (product.old_price
                                            ? '<span class="old-price">' +
                                                  '{0} {1}'.format(product.old_price, getLocalizedCurrency(config.storeCurrency)) +
                                              '</span>'
                                            : '') +
                                    '</div>' +
                                '</div>' +
                            '</div>' +
                            '<div class="right col-40" id="prodInfo">' +
                                '<a href="#" id="btnCart" class="button ui_button_text ui_button_background">' +
                                    '{0}'.format(l10n.store[lang].add_to_cart) +
                                '</a>' +

                                '<div class="cart-item-price" style="display: none">' +
                                    '<div class="buttons-row ui_button_background" style="border:1px solid #ccc;border-radius:5px">' +
                                        '<a href="#" id="btnMinus" class="button">-</a>' +
                                        '<span class="button" id="qtycart" style="border-left:1px solid #ccc;border-right:1px solid #ccc">' +
                                            '0' +
                                        '</span>' +
                                        '<a href="#" id="btnPlus" class="button">+</a>' +
                                    '</div>' +
                                '</div>' +
                            '</div>' +
                        '</div>' +
                    '</div>' +
                '</div>' +
                '<div class="content-block-title">' +
                    '<div class="tabbar">' +
                        '<div class="toolbar-inner">' +
                            '<a href="#tab1" class="tab-link active" ' + 'style="color: ' + (config.settings.characteristics_color || '#1EAE0E') + ';">' +
                                description +
                            '</a>' +
                            '<a href="#tab2" id="charsTab" class="tab-link" ' + 'style="color: ' + (config.settings.description_color || '#1EAE0E') + ';">' +
                                charsTitle +
                            '</a>' +
                        '</div>' +
                    '</div>' +
                '</div>' +
                '<div class="list-block list-block_background">' +
                    '<ul class="card-ul">' +
                        '<li class="accordion-item">' +
                            '<div class="accordion-item-content" style="height: auto;">' +
                                '<div class="tabs content-block ">' +
                                    '<div id="tab1" class="tab active">' +
                                        '<p>{0}<br></p>'.format(product.description) +
                                    '</div>' +
                                    '<div id="tab2" class="tab">' +
                                        '<table class="table-chars">' +
                                            '<tbody id="charsBody"></tbody>' +
                                        '</table>' +
                                    '</div>' +
                                '</div>' +
                            '</div>' +
                        '</li>' +
                        '<li>' +
                            '<div class="item-content item-link">' +
                                '<a href="#" id="shareBtn" class="link ui_text_color">' +
                                    '<i class="fa fa-share-alt" style="padding:0 5px;font-size:18px;"></i>' +
                                    shareTitle +
                                '</a>' +
                                '<a href="#" class="link ui_text_color favourites">' +
                                    '<i class="fa fa-heart"></i>' +
                                    '{0}'.format(l10n.store[lang].to_favorites) +
                                '</a>' +
                            '</div>' +
                        '</li>' +
                    '</ul>' +
                '</div>' +
            '</div>';

        var $card = $(template);
        // необходимо добавлять класс external иначе ссылки открываются через Ajax
        // что приводит к ошибке
        $card.find('#tab1').find('a').addClass('external');
        setImageSlides($card, product);

        var $tbody = $card.find('#charsBody');
        var chars = $.parseJSON(product.characteristics);

        for(var i = 0; i < chars.length; i++) {
            $tbody.append($('<tr><td>{0}</td><td>{1}</td></tr>'.format(chars[i].title, chars[i].value)));
        }

        $card.find('#btnCart').click({product: product}, addToCartFunc);
        $card.find('#btnMinus').click({product: product}, removeCardFromCard);
        $card.find('#btnPlus').click({product: product}, addToCartFunc);

        if(!config.settings.showSocial) {
            $card.find('#shareBtn').hide();
        }

        if(!config.settings.showPrices) {
            $card.find('#prdPrice').hide();
        }

        if(!config.settings.showModifications) {
            $card.find('#charsTab').hide();
        }

        if(!config.settings.showCart) {
            $card.find('#btnCart').hide();
        }

        if (config.settings.showRelatedProducts
            && product.related_products
            && product.related_products.length > 0
        ) {
            var $swiper = renderRelatedProductsSlider(product.related_products);
            $card.find('.card-ul')
                .after($swiper)
                .after(
                    '<div class="related-product-label">' +
                        l10n.store[lang].also_buy_with_product +
                    '</div>'
                );
        }

        var favoritesLink = $card.find('.favourites');
        var shareLink = $card.find('#shareBtn');

        if($.inArray(product.id, favorites) > -1) {
            favoritesLink.css('color', favoritesColor);
        }

        favoritesLink.click({productId: product.id}, favoritesClickFunc);
        shareLink.click(function() {
            copyStringToClipboard(
                schema + '://' + serverUrl +
                '/shop/show-product?product_id=' + product.id
            );
            myApp.alert(l10n.store[lang].link_copied);
        });

        if(product.basket_count > 0) {
            $card.find('#btnCart').css('display', 'none');
            $card.find('.cart-item-price').css('display', 'block');
            $card.find('#qtycart').text(product.basket_count);
        }

        self.visualElement.empty();
        self.visualElement.append($card);

        forceRedraw($('.related-products-wrapper').get(0));
    };


    /**
     * Returns swiper object with related products.
     *
     * @param {object} products   Products to render swiper with.
     *
     * @return {jQuery}   Swiper object.
     */
    function renderRelatedProductsSlider(products) {
        var $swiper = $(
            '<div class="related-products">' +
                '<div class="related-products-wrapper"></div>' +
            '</div>'
        );
        var $wrapper = $swiper.find('.related-products-wrapper');

        for (var i = 0; i < products.length; i++) {
            var image;
            try {
                image = (JSON.parse(products[i].image))[0];
            } catch (e) {
                return $('');
            }

            $wrapper.append($(
                '<a href="#" data-product_id="' + products[i].id + '" ' +
                   'class="related-product">' +
                    (image ? '<img src="' + image + '"/>' : '') +
                    '<div>' + cutName(products[i].name) + '</div>' +
                '</a>'
            ));
        }
        return $swiper;


        /**
         * Cuts given text for up to 8 symbols.
         *
         * @param {string} text   Text to cut.
         *
         * @return {string}   Cut text.
         */
        function cutName(text) {
            if (text.length <= 9) {
                return text;
            }
            return text.substr(0, 9) + '...';
        }
    }

    /**
     * Установить изображения товара в слайдер
     *
     * @param {object} $card - jQuery объект - карточка товара.
     * @param {object} product - данные товара.
     */
    function setImageSlides($card, product) {
        var images = [];

        if(product.image !== undefined && product.image !== null && product.image !== '') {
            images = $.parseJSON(product.image);
        }

        needSwiper = images.length > 0;

        var $swiper = $card.find('.swiper-wrapper');

        for(var i = 0; i < images.length; i++) {
            var template = '<div class="swiper-slide">' +
                                '<div class="offer-item-img" style="padding: 0;">' +
                                    '<div class="fa fa-search"></div>' +
                                '</div>' +
                            '</div>';

            var $slide = $(template);
            $slide.find('.offer-item-img')
                  .css('background-image', 'url("{0}")'.format(images[i]))
                  .data('image', images[i]);

            $swiper.append($slide);

            $slide.find('.offer-item-img .fa-search').click(function() {
                if (!isProduction()) {
                    alert(l10n.store[lang].zoom_work_only_on_phones);
                    return;
                }
                var $iframe = $('<iframe class="iframe-store-product-modal">');

                var html = '' +
                    '<head>' +
                        '<style>' +
                            '.store-product-modal-content {' +
                            '    position: fixed;' +
                            '    top: 30px;' +
                            '    bottom: 30px;' +
                            '    left: 0;' +
                            '    width: 100%;' +
                            '    overflow: auto;' +
                            '}' +
                            '.store-product-modal-content img {' +
                            '    width: 150%;' +
                            '    height: auto;' +
                            '}' +
                        '</style>' +
                    '</head>' +
                    '<body>' +
                        '<div class="store-product-modal-content">' +
                            '<img src="' + $(this).parent().data('image') + '"/>' +
                        '</div>' +
                    '</body>';

                $iframe.src = 'javascript:void(0);';
                $iframe.load(function() {
                    $(this).off('load');
                    $iframe.contents().find('html').html(html);
                });
                $('body').append($iframe)
                         .append($('<span class="store-product-close">&times;</span>'));
                $('.store-product-close').click(function() {
                    $iframe.remove();
                    $(this).remove();
                });
            });
        }
    }

    /**
     * Инициализация слайдера
     */
    this.initSwiper = function() {
        if(!needSwiper) {
            return;
        }

        myApp.swiper('.swiper-container', {
            pagination:'.swiper-pagination',
            spaceBetween: 10
        });
    };
}

/**
 * Класс для работы с корзиной
 */
function ShopCart() {
    var self = this;
    this.visualElement = null;
    var config = null;
    var $emptyDiv = null;
    var deviceId = null;
    var instanceId = null;
    var cartItems = null;
    var productClickMethod = null;
    var orderClickMethod = null;

    /**
     * Инициализация корзины.
     * @param {int} shopId - instanceId магазина.
     * @param {int} userId - deviceId устройства.
     * @param {object} storeConfig - конфиг магазина.
     * @param {function} productClick - делегат, метод для клика на иконке товара.
     * @param {function} orderClick - делегат, метод для клика на кнопке заказа.
     */
    this.init = function(shopId, userId, storeConfig, productClick, orderClick) {
        config = storeConfig;
        instanceId = shopId;
        deviceId = userId;
        productClickMethod = productClick;
        orderClickMethod = orderClick;
        createVisualElement();
    };

    /**
     * Создание основного визуального элемента корзины
     */
    function createVisualElement() {
        var template = '<div class="page ui_text_color page-on-center" style="display: none">' +
                            '<div class="page-content cart">' +
                                '<div id="cartItems">' +
                                    '<div class="content-block">' +
                                    '</div>' +
                                    '<div id="emptyDiv" class="category-list" style="display: none">' +
                                        '<div style="padding-left: 25px" class="">' +
                                            '{0}'.format(l10n.store[lang].cart_empty) +
                                        '</div>' +
                                    '</div>' +
                                '</div>' +
                            '</div>' +
                            '<div class="toolbar toolbar-bottom ui_price_layout_background">' +
                                '<div class="toolbar-inner">' +
                                    '<div class="left">' +
                                        '<div class="total ui_text_color_secondary">' +
                                            '<div class="total-qty"></div>' +
                                            '<div class="total-price"></div>' +
                                        '</div>' +
                                    '</div>' +
                                    '<div class="right">' +
                                        '<a href="#" class="button ui_button_text ui_button_background">' +
                                            l10n.store[lang].place_an_order +
                                        '</a>' +
                                    '</div>' +
                                '</div>' +
                            '</div>' +
                        '</div>';

        self.visualElement = $(template);
        self.visualElement.find('.toolbar .right a').click(function() {
            var count = 0;
            var sum = 0.0;

            for (var i = 0; i < cartItems.length; i++) {
                var cartItem = cartItems[i];
                count += cartItem.count;
                sum += cartItem.product.price * cartItem.count;
            }

            if(count === 0) {
                myApp.alert(l10n.store[lang].cart_empty);
                return;
            }

            orderClickMethod(count, sum);
        });

        if(!config.settings.showPrices) {
            self.visualElement.find('.total-price').css('display', 'none');
        }

        $emptyDiv = self.visualElement.find('#emptyDiv');
    }

    /**
     * Добавление\удаление\измненеия кол-ва товаров в корзине
     * @param {int} productId - id товара.
     * @param {int} count - кол-во товара.
     * @param {function} callback - делегат, метод, который вызывается при удачном запросе с сервера.
     */
    this.updateCart = function(productId, count, callback) {
        if(callback === null || callback === undefined) {
            callback = function() {};
        }

        $.post(schema + '://' + serverUrl + '/shop/update-basket',
            {
                instance_id: instanceId,
                device_id: deviceId,
                product_id: productId,
                product_count: count
            }, function() {
                callback();
                updateBasketCount();
            })
            .fail(function(xhr, status, error) {
                myApp.alert(status);
            });
    };

    /**
     * Обновление корзины (получение содержимого)
     */
    this.refreshCart = function() {
        $.get(schema + '://' + serverUrl + '/shop/get-basket-items',
            {
                instance_id: instanceId,
                device_id: deviceId
            }, refreshData)
            .fail(function(xhr, status, error) {
                myApp.alert(status);
            });
    };

    /**
     * Отображение содержимого текущей категории.
     *
     * @param {string} data - данные в JSON формате.
     */
    function refreshData(data) {
        cartItems = $.parseJSON(data);

        var $contentBlock = self.visualElement.find('.content-block');
        $contentBlock.empty();
        $emptyDiv.hide();

        for (var i = 0; i < cartItems.length; i++) {
            var cartItem = cartItems[i];
            $contentBlock.append(createProductCard(cartItem));
        }

        updateFooter();
    }

    /**
     * Создать HTML представление карточки товара.
     *
     * @param {object} item - данные товара в корзине.
     * @return {object} jQuery объект - карточка товара.
     */
    function createProductCard(item) {
        var template = '<div class="cart-item">' +
                            '<div class="row no-gutter cart-item-container">' +
                                '<div class="cart-item-info col-33">' +
                                    '<div class="catalog-item-img-div">' +
                                        '<div class="catalog-item-img"></div>' +
                                    '</div>' +
                                '</div>' +
                                '<div class="cart-item-info col-50">' +
                                    '<div class="catalog_item">' +
                                        '{0}'.format(item.product.name) +
                                    '</div>' +
                                '</div>' +
                                '<div class="cart-item-info col-10">' +
                                    '<a href="#" id="btnRemove" class="link icon-only">' +
                                        '<i class="fa fa-trash ui_text_color" style="font-size:24px"></i>' +
                                    '</a>' +
                                '</div>' +
                            '</div>' +
                            '<div class="row no-gutter cart-item-container">' +
                                '<div class="col-auto">' +
                                    '<div id="itemPrice" class="cart-item-price bold">' +
                                        '{0} {1}'.format(item.product.price, getLocalizedCurrency(config.storeCurrency)) +
                                    '</div>' +
                                '</div>' +
                                '<div class="col-40">' +
                                    '<div class="cart-item-price">' +
                                        '<div class="buttons-row ui_button_background" style="border:1px solid #ccc;border-radius:5px">' +
                                            '<a href="#" id="btnMinus" class="button">-</a>' +
                                            '<span class="button" id="qtycart1" style="border-left:1px solid #ccc;border-right:1px solid #ccc">' +
                                                '{0}'.format(item.count) +
                                            '</span>' +
                                            '<a href="#" id="btnPlus" class="button">+</a>' +
                                        '</div>' +
                                    '</div>' +
                                '</div>' +
                                '<div class="col-auto">' +
                                    '<div class="cart-item-price bold right" id="totalPrice">' +
                                        '{0} {1}'.format(item.product.price * item.count, getLocalizedCurrency(config.storeCurrency)) +
                                    '</div>' +
                                '</div>' +
                            '</div>' +
                        '</div>';

        var $card = $(template);
        $card.data('productId', item.product.id);
        $card.find('#btnRemove').click({cartItem: item}, removeProductWarning);
        $card.find('#btnMinus').click({cartItem: item}, decrementProduct);
        $card.find('#btnPlus').click({cartItem: item}, incrementProduct);
        $card.find('.catalog-item-img-div').click({productId: item.product.id}, productClickMethod);

        if(!config.settings.showPrices) {
            $card.find('#itemPrice').css('display', 'none');
            $card.find('#totalPrice').css('display', 'none');
        }

        setImage($card, item.product);

        return $card;
    }

    /**
     *  Увеличивает кол-во товара в корзине на 1
     *
     * @param {object} event - event объект события.
     */
    function incrementProduct(event) {
        var item = event.data.cartItem;

        var callback = function() {
            updateCartItem(item, item.count + 1);
        };

        self.updateCart(item.product.id, item.count + 1, callback);
    }

    /**
     *  Уменьшает кол-во товара в корзине на 1
     *
     * @param {object} event - event объект события.
     */
    function decrementProduct(event) {
        var item = event.data.cartItem;

        if((item.count - 1) === 0) {
            var obj = {data: {cartItem: item}};
            removeProductWarning(obj);
            return;
        }

        var callback = function() {
            updateCartItem(item, item.count - 1);
        };

        self.updateCart(item.product.id, item.count - 1, callback);
    }

    /**
     * Обновляет информацию в футере
     */
    function updateFooter() {
        var count = 0;
        var sum = 0.0;

        for (var i = 0; i < cartItems.length; i++) {
            var cartItem = cartItems[i];
            count += cartItem.count;
            sum += cartItem.product.price * cartItem.count;
        }

        self.visualElement.find('.total-qty').text('{0} {1}'.format(count, l10n.store[lang].pc));
        self.visualElement.find('.total-price').text('{0} {1}'.format(sum, getLocalizedCurrency(config.storeCurrency)));

        if(count === 0) {
            $emptyDiv.show();
        }
    }

    /**
     * Обновляет визуальный элемент корзины
     * @param {object} item - данные товара в корзине.
     * @param {int} newCount - кол-во данного товара в корзине.
     */
    function updateCartItem(item, newCount) {
        item.count = newCount;

        var $div = findProductDiv(item.product.id);
        $div.find('#qtycart1').text(item.count);
        $div.find('#totalPrice').text('{0} {1}'.format(item.product.price * item.count, getLocalizedCurrency(config.storeCurrency)));

        updateFooter();
    }

    /**
     * Находит элемент корзины по id товара
     * @param {int} productId -  id товара
     * @return {object} jQuery представление карточки товара
     */
    function findProductDiv(productId) {
         var divs = self.visualElement.find('.cart-item');

        for(var i = 0; i < divs.length; i++) {
            var $div = $(divs[i]);

            if($div.data('productId') === productId) {
                return $div;
            }
        }

        return null;
    }

    /**
     * Удаление товара с предупреждением
     * @param {object} event - event объект события.
     */
    function removeProductWarning(event) {
        var item = event.data.cartItem;

        myApp.modal({
            text: l10n.store[lang].cart_item_remove_confirm,
            buttons: [
                {
                    text: l10n.store[lang].cancel
                },
                {
                    text: l10n.store[lang].remove,
                    onClick: function() {
                        removeProduct(item);
                    }
                }
            ]
        });
    }

    /**
     * Удаление товара из корзины
     *
     * @param {object} item - элемент корзины
     */
    function removeProduct(item) {
        var productId = item.product.id;

        var callback = function() {
            item.count = 0;
            removeVisualProduct(productId);
            updateFooter();
        };

        self.updateCart(productId, 0, callback);
    }

    /**
     * Удаление визуального элемента товара из корзины
     *
     * @param {int} productId -  id товара
     */
    function removeVisualProduct(productId) {
        var $div = findProductDiv(productId);
        $div.remove();
    }
}

/**
* Класс для работы с формированием заказа
*/
function ShopOrder() {
    var self = this;
    this.visualElement = null;
    var $orderElement = null;
    var $additionalFieldsElement = null;
    var deviceId = null;
    var instanceId = null;
    var deliveries = null;
    var regions = null;
    var config = null;
    var createOrderAction = null;
    var lastDelivery = null;
    var sumOrder;
    var countProducts;

    /**
     * Инициализация заказа.
     * @param {int} shopId - instanceId магазина.
     * @param {int} userId - deviceId устройства.
     * @param {object} shopConfig - конфиг магазина.
     * @param {function} onCreateOrder - делегат, выполняемый при создании заказа.
     */
    this.init = function(shopId, userId, shopConfig, onCreateOrder) {
        instanceId = shopId;
        deviceId = userId;
        config = shopConfig;
        createOrderAction = onCreateOrder;
        createVisualElement();
    };

    /**
     * Назад. В пределах оформления заказа.
     *
     * @return {boolean} успешно ли произвелся возврат назад.
     */
    this.back = function() {
        if($additionalFieldsElement !== null && $additionalFieldsElement.parent().length) {
            var $page = self.visualElement.find('.page-content');
            $page.empty();
            $page.append($orderElement);

            var $orderBtn = self.visualElement.find('.toolbar .right a');
            $orderBtn.off('click');
            $orderBtn.click(orderClick);
            $orderBtn.addClass('disabled');

            checkDataForEnabledOrder();

            return true;
        }

        return false;
    };

    /**
     * Создание основного визуального элемента заказа
     */
    function createVisualElement() {
        var template = '<div class="page ui_text_color page-on-center" style="display: none">' +
                            '<div class="page-content">' +
                            '</div>' +
                            '<div class="toolbar toolbar-bottom ui_price_layout_background">' +
                                '<div class="toolbar-inner">' +
                                    '<div class="left">' +
                                        '<div class="total ui_text_color_secondary">' +
                                            '<div class="total-qty"></div>' +
                                            '<div class="total-price"></div>' +
                                        '</div>' +
                                    '</div>' +
                                    '<div class="right">' +
                                        '<a href="#" class="button ui_button_text ui_button_background disabled">' +
                                            l10n.store[lang].place_an_order +
                                        '</a>' +
                                    '</div>' +
                                '</div>' +
                            '</div>' +
                        '</div>';

        self.visualElement = $(template);
        self.visualElement.find('.toolbar .right a').click(orderClick);

        if(!config.settings.showPrices) {
            self.visualElement.find('.total-price').css('display', 'none');
        }
    }

    /**
     * Создание формы заполнения контактной информации
     *
     * @return {object} jQuery представление формы заполнения контактной информации
     */
    function createContactInfoElement() {
        var template = '<div class="orderPage">' +
                            '<div class="content-block-title">{0}</div>'.format(l10n.store[lang].contact_information) +
                            '<div class="list-block">' +
                                '<ul class="order-fields">' +
                                    '<li class="order-field-required">' +
                                        '<div class="item-content">' +
                                            '<div class="item-inner">' +
                                                '<div class="item-input">' +
                                                    '<input id="nameInput" type="text" placeholder="{0}">'.format(l10n.store[lang].full_name) +
                                                '</div>' +
                                            '</div>' +
                                        '</div>' +
                                    '</li>' +
                                    '<li class="order-field-required">' +
                                        '<div class="item-content">' +
                                            '<div class="item-inner">' +
                                                '<div class="item-input">' +
                                                    '<input id="phoneInput" type="tel" placeholder="{0}">'.format(l10n.store[lang].phone) +
                                                '</div>' +
                                            '</div>' +
                                        '</div>' +
                                    '</li>' +
                                    '<li>' +
                                        '<div class="item-content">' +
                                            '<div class="item-inner">' +
                                                '<div class="item-input">' +
                                                    '<input id="emailInput" type="email" placeholder="E-mail">' +
                                                '</div>' +
                                            '</div>' +
                                        '</div>' +
                                    '</li>' +
                                    '<li>' +
                                        '<div class="item-content">' +
                                            '<div class="item-inner">' +
                                                '<div class="item-input">' +
                                                    '<input id="additionalInput" type="text" placeholder="{0}">'.format(l10n.store[lang].additionally) +
                                                '</div>' +
                                            '</div>' +
                                        '</div>' +
                                    '</li>' +
                                '</ul>' +
                            '</div>' +
                            '<div class="content-block-title">{0}</div>'.format(l10n.store[lang].region) +
                            '<div class="list-block">' +
                                '<ul>' +
                                    '<li>' +
                                        '<div class="item-content">' +
                                            '<div class="item-inner">' +
                                                '<div class="item-input">' +
                                                    '<select id="regions_select">' +
                                                        '<option value="?">{0}</option>'.format(l10n.store[lang].select_region) +
                                                    '</select>' +
                                                '</div>' +
                                            '</div>' +
                                        '</div>' +
                                    '</li>' +
                                '</ul>' +
                            '</div>' +
                            '<div id="deliveriesDiv" style="display: none">' +
                                '<div class="content-block-title">{0}</div>'.format(l10n.store[lang].delivery) +
                                '<div class="list-block">' +
                                    '<ul>' +
                                        '<li>' +
                                            '<div class="item-content">' +
                                                '<div class="item-inner">' +
                                                    '<div class="item-input">' +
                                                        '<select id="deliveries_select">' +
                                                            '<option value="?">{0}</option>'.format(l10n.store[lang].select_delivery_method) +
                                                        '</select>' +
                                                    '</div>' +
                                                '</div>' +
                                            '</div>' +
                                        '</li>' +
                                    '</ul>' +
                                '</div>' +
                            '</div>' +
                            '<div id="paymentsDiv" style="display: none">' +
                                '<div class="content-block-title">{0}</div>'.format(l10n.store[lang].payment) +
                                '<div class="list-block">' +
                                    '<ul>' +
                                        '<li>' +
                                            '<div class="item-content">' +
                                                '<div class="item-inner">' +
                                                    '<div class="item-input">' +
                                                        '<select id="payments_select">' +
                                                            '<option value="?">{0}</option>'.format(l10n.store[lang].select_payment_method) +
                                                        '</select>' +
                                                    '</div>' +
                                                '</div>' +
                                            '</div>' +
                                        '</li>' +
                                    '</ul>' +
                                '</div>' +
                            '</div>' +
                        '</div>';

        return $(template);
    }

    /**
     * Создание формы заполнения контактной информации
     * @param {object} delivery - объект с данными способа доставки.
     * @return {object} jQuery представление формы заполнения контактной информации
     */
    function createAdditionalInfoElement(delivery) {
        var template = '<div class="orderPage">' +
                            '<div class="content-block-title">' + l10n.store[lang].contact_information + '</div>' +
                            '<div class="list-block">' +
                                '<ul></ul>' +
                            '</div>' +
                        '</div>';

        var $mainDiv = $(template);
        var $ul = $mainDiv.find('ul');

        for(var i = 0; i < delivery.fields.length; i++) {
            var liTemplate = '<li>' +
                                '<div class="item-content">' +
                                    '<div class="item-inner">' +
                                        '<div class="item-input">' +
                                            '<input id="field{0}" type="text" placeholder="{1}">'.format(i, delivery.fields[i].title) +
                                        '</div>' +
                                    '</div>' +
                                '</div>' +
                            '</li>';

            $ul.append(liTemplate);
            $ul.find('#field' + i).change(checkAdditionalDataForEnabledOrder);
        }

        return $mainDiv;
    }

    /**
     * Установка футера
     *
     * @param {int} count - кол-во товаров.
     * @param {int} sum - стоимость всех товаров.
     */
    this.setFooter = function(count, sum) {
        countProducts = count;
        sumOrder = sum;
        self.visualElement.find('.total-qty').text('{0} {1}'.format(count, l10n.store[lang].pc));
        self.visualElement.find('.total-price').text('{0} {1}'.format(sum, getLocalizedCurrency(config.storeCurrency)));
    };

    /**
     * Обновление заказа (получение данных о доставках и оплате)
     */
    this.refreshOrder = function() {
        $.get(schema + '://' + serverUrl + '/shop/get-delivery-types',
            {
                instance_id: instanceId
            }, function(data) {
                deliveries = $.parseJSON(data);

                for(var i = 0; i < deliveries.length; i++) {
                    deliveries[i].fields = $.parseJSON(deliveries[i].fields);
                }

                loadRegions();
            })
            .fail(function(xhr, status, error) {
                myApp.alert(status);
            });
    };

    /**
     * Получение регионов
     */
    function loadRegions() {
        $.get(schema + '://' + serverUrl + '/shop/get-regions',
            {
                instance_id: instanceId
            }, function(data) {
                filterRegionByDeliveries(data);
                refreshData();
            })
            .fail(function() {
                myApp.alert(status);
            });
    }

    /**
     * Фильтрация регионов привязанных к доставкам
     *
     * @param {string} data - данные в JSON формате.
     */
    function filterRegionByDeliveries(data) {
        var allRegions = $.parseJSON(data);
        regions = [];

        for(var r = 0; r < allRegions.length; r++) {
            var region = allRegions[r];

            for(var d = 0; d < deliveries.length; d++) {
                var dRegions = deliveries[d].regions.split(',');

                if($.inArray(region.id.toString(), dRegions) > -1) {
                    regions.push(region);
                    break;
                }
            }
        }
    }

    /**
     * Обновление заказа (визуальное)
     */
    function refreshData() {
        lastDelivery = null;
        $additionalFieldsElement = null;

        $orderElement = createContactInfoElement();
        var $regionsSelect = $orderElement.find('#regions_select');
        var $page = self.visualElement.find('.page-content');
        $page.empty();

        for (var i = 0; i < regions.length; i++) {
            var region = regions[i];
            var $option = $('<option value="{0}">{1}</option>'.format(region.id, region.name));
            $regionsSelect.append($option);
        }

        $orderElement.find('#nameInput').blur(highLightField);
        $orderElement.find('#phoneInput').blur(highLightField);

        $orderElement.find('#nameInput').change(checkDataForEnabledOrder);
        $orderElement.find('#phoneInput').change(checkDataForEnabledOrder);
        $orderElement.find('#emailInput').change(checkDataForEnabledOrder);

        $regionsSelect.change(regionTypeChanged);
        $orderElement.find('#deliveries_select').change(deliveryTypeChanged);
        $orderElement.find('#payments_select').change(checkDataForEnabledOrder);

        $page.append($orderElement);


        /**
         * Performs highlighting given field if it is empty.
         */
        function highLightField() {
            var $this = $(this);
            if (!$this.val()) {
                $this.parents('li.order-field-required').addClass('invalid');
            } else {
                $this.parents('li.order-field-required').removeClass('invalid');
            }
            checkDataForEnabledOrder();
        }
    }

    /**
     * Обработчик выбора региона.
     */
    function regionTypeChanged() {
        var $option = self.visualElement.find('#regions_select option:selected');
        var $deliveriesDiv = self.visualElement.find('#deliveriesDiv');
        var $paymentsDiv = self.visualElement.find('#paymentsDiv');
        var $deliverySelect = $deliveriesDiv.find('select');
        $deliverySelect.empty();
        $deliverySelect.append('<option value="?">{0}</option>'.format(l10n.store[lang].select_delivery_method));

        if($option.val() === '?') {
            $deliveriesDiv.hide();
            $paymentsDiv.hide();
        } else {
            for(var i = 0; i < deliveries.length; i++) {
                var delivery = deliveries[i];
                var regionId = $option.val();

                if ($.inArray(regionId, delivery.regions.split(',')) > -1) {
                    var deliveryPrice = 0;

                    if(delivery.price_type === 0) {
                        deliveryPrice = delivery.price;
                    } else if(delivery.price_type === 1) {
                        deliveryPrice = delivery.price * countProducts;
                    }

                    if(delivery.free_delivery_price !== undefined &&
                        delivery.free_delivery_price !== 0 &&
                        sumOrder >= delivery.free_delivery_price) {
                        deliveryPrice = 0;
                    }

                    var deliveryName = delivery.name;

                    if(deliveryPrice !== 0) {
                        deliveryName += ' - {0} {1}'.format(deliveryPrice, getLocalizedCurrency(config.storeCurrency));
                    }

                    var $newOption = $('<option value="{0}">{1}</option>'.format(delivery.id, deliveryName));
                    $deliverySelect.append($newOption);
                }
            }

            $deliveriesDiv.show();
        }

        deliveryTypeChanged();

        checkDataForEnabledOrder();
    }

    /**
     * Обработчик выбора метода доставки.
     */
    function deliveryTypeChanged() {
        var $option = self.visualElement.find('#deliveries_select option:selected');
        var $paymentDiv = self.visualElement.find('#paymentsDiv');
        var $paymentSelect = $paymentDiv.find('select');
        $paymentSelect.empty();
        $paymentSelect.append('<option value="?">{0}</option>'.format(l10n.store[lang].select_payment_method));

        if($option.val() === '?') {
            $paymentDiv.hide();
            self.visualElement.find('.total-price').text('{0} {1}'.format(sumOrder, getLocalizedCurrency(config.storeCurrency)));
        } else {
            var delivery = findDelivery(parseInt($option.val()));

            var deliveryPrice = 0;

            if(delivery.price_type === 0) {
                deliveryPrice = delivery.price;
            } else if(delivery.price_type === 1) {
                deliveryPrice = delivery.price * countProducts;
            }

            if(delivery.free_delivery_price !== undefined &&
                delivery.free_delivery_price !== 0 &&
                sumOrder >= delivery.free_delivery_price) {
                deliveryPrice = 0;
            }

            self.visualElement.find('.total-price').text('{0} {1}'.format(sumOrder + deliveryPrice, getLocalizedCurrency(config.storeCurrency)));

            for(var i = 0; i < config.payments.length; i++) {
                var payment = config.payments[i];

                if (delivery.payment_type.indexOf(payment.id + ';') >= 0) {
                    var $newOption;

                    if(payment.id === 'cash') {
                        $newOption = $('<option value="{0}">{1}</option>'.format(payment.id, l10n.store[lang].cash));
                    } else if(payment.id === 'courierCard') {
                        $newOption = $('<option value="{0}">{1}</option>'.format(payment.id, l10n.store[lang].courierCard));
                    } else if(sumOrder !== 0) {
                        $newOption = $('<option value="{0}">{1}</option>'.format(payment.id, payment.name));
                    }

                    $paymentSelect.append($newOption);
                }
            }

            $paymentDiv.show();
        }

        checkDataForEnabledOrder();
    }

    /**
     * Найти способ доставки по его id.
     *
     * @param {int} deliveryId - id способа доставки.
     * @return {object} объект способа доставки.
     */
    function findDelivery(deliveryId) {
        for (var i = 0; i < deliveries.length; i++) {
            var delivery = deliveries[i];

            if (delivery.id === deliveryId) {
                return delivery;
            }
        }

        return null;
    }

    /**
     * Проверяет данные для блокировки-разблокировки кнопки оформления заказа
     */
    function checkDataForEnabledOrder() {
        var $nameInput = self.visualElement.find('#nameInput');
        var name = $nameInput.val();
        var phone = self.visualElement.find('#phoneInput').val();
        // var email = self.visualElement.find('#emailInput').val();

        var region = self.visualElement.find('#regions_select option:selected').val();
        var delivery = self.visualElement.find('#deliveries_select option:selected').val();
        var payment = self.visualElement.find('#payments_select option:selected').val();

        var result = true;

        if(name === undefined || name === '' ||
            phone === undefined || phone === '' ||
            // email === undefined || email === '' ||
            region === '?' || delivery === '?' || payment === '?') {
            result = false;
        }

        var $orderBtn = self.visualElement.find('.toolbar .right a');

        if(result) {
            $orderBtn.removeClass('disabled');
        } else {
            $orderBtn.addClass('disabled');
        }
    }

    /**
     * Проверяет жполнительные данные для блокировки-разблокировки кнопки оформления заказа
     */
    function checkAdditionalDataForEnabledOrder() {
        var result = true;

        for(var i = 0; i < lastDelivery.fields.length; i++) {
            var value = $additionalFieldsElement.find('#field' + i).val();

            if(lastDelivery.fields[i].isRequired && (value === undefined || value === '')) {
                result = false;
            }
        }

        var $orderBtn = self.visualElement.find('.toolbar .right a');

        if(result) {
            $orderBtn.removeClass('disabled');
        } else {
            $orderBtn.addClass('disabled');
        }
    }

    /**
     * Непосредственное оформление заказа
     */
    function createOrder() {
        var name = $orderElement.find('#nameInput').val();
        var phone = $orderElement.find('#phoneInput').val();
        var email = $orderElement.find('#emailInput').val();
        var additional = $orderElement.find('#additionalInput').val();

        var region = $orderElement.find('#regions_select option:selected').val();
        var delivery = $orderElement.find('#deliveries_select option:selected').val();
        var payment = $orderElement.find('#payments_select option:selected').val();

        var fields = [];

        if(lastDelivery !== null) {
            for(var i = 0; i < lastDelivery.fields.length; i++) {
                var value = $additionalFieldsElement.find('#field' + i).val();

                var field = {
                    title: lastDelivery.fields[i].title,
                    value: value
                };

                fields.push(field);
            }
        }

        $.post(schema + '://' + serverUrl + '/shop/create-order',
            {
                instance_id: instanceId,
                device_id: deviceId,
                region_id: region,
                delivery_id: delivery,
                payment_type: payment,
                delivery_properties: JSON.stringify(fields),
                fio: name,
                phone: phone,
                email: email,
                additional: additional
            }, function(data) {
                if(data == 0) {
                    myApp.alert(l10n.store[lang].something_went_wrong);
                } else {
                    showOrderSuccess($.parseJSON(data).id, payment);
                }
            })
            .fail(function() {
                myApp.alert(status);
            });
    }

    /**
     * Обработчик нажатия на кнопку оформления заказа
     */
    function orderClick() {
        var $option = self.visualElement.find('#deliveries_select option:selected');
        var delivery = findDelivery(parseInt($option.val()));

        if(delivery.fields !== null && delivery.fields !== undefined &&
            delivery.fields.length > 0) {
            $additionalFieldsElement = createAdditionalInfoElement(delivery);
            lastDelivery = delivery;

            var $page = self.visualElement.find('.page-content');
            $page.empty();
            $page.append($additionalFieldsElement);

            var $orderBtn = self.visualElement.find('.toolbar .right a');
            $orderBtn.off('click');
            $orderBtn.click(createOrder);
            $orderBtn.addClass('disabled');
            checkAdditionalDataForEnabledOrder();
        } else {
            createOrder();
        }
    }

    /**
     * Отобразить страницу успешного создания заказа
     *
     * @param {int} orderId - id заказа.
     * @param {string} payment - тип платежа.
     */
    function showOrderSuccess(orderId, payment) {
        if (payment !== 'paypal') {
            renderPage();
            return;
        }

        getPaypalToken(orderId).done(function(token) {
            var $page = renderPage();
            var action = config.paypal_api_entrypoint +
                            '?cmd=_express-checkout&token=' + token;
            var $paymentLink = $(
                '<a href="' + action + '" target="_blank" ' +
                   'class="external button ui_button_text ui_button_background">' +
                    l10n.store[lang].order_pay +
                '</a>'
            );
            $paymentLink.click(function() {
                $(this).remove();
            });
            $page.append(
                $('<div class="content-block"></div>').append($paymentLink)
            );
        }).fail(function() {
            renderPage();
            myApp.alert(l10n.store[lang].something_went_wrong);
        });


        /**
         * Renders page with success order message.
         *
         * @return {object}   Generated page as jQuery object.
         */
        function renderPage() {
            if (createOrderAction !== null) {
                createOrderAction();
            }

            var template =
                '<div class="content-block">' +
                    '<div style="word-wrap: break-word; width: 100%">' +
                        l10n.store[lang].thx_for_order +
                    '</div>' +
                '</div>' +
                '<div class="content-block">' +
                    '<a id="btnHome" href="#" ' +
                       'class="button ui_button_text ui_button_background">' +
                        l10n.store[lang].to_main_screen +
                    '</a>' +
                '</div>' +
                '<div class="content-block">' +
                    l10n.store[lang].order_number.format(orderId) +
                '</div>';

            var $element = $(template);
            $element.find('#btnHome').click(function() {
                mainView.router.load({url: 'index.html'});
            });

            self.visualElement .find('.toolbar-bottom').hide();
            var $page = self.visualElement.find('.page-content');
            $page.empty();
            $page.append($element);

            updateBasketCount();
            return $page;
        }
    }

    /**
     * Returns paypal token.
     *
     * @param {int} orderId   Order ID to return token for.
     *
     * @return {object}   Deferred with token.
     */
    function getPaypalToken(orderId) {
        var $deferred = new $.Deferred();
        $.get(schema + '://' + serverUrl + '/paypal/get-token', {
            order_id: orderId
        }).done(function(data) {
            try {
                var result = $.parseJSON(data);
            } catch (e) {
                $deferred.reject();
            }
            if (result === 'error') {
                $deferred.reject();
            } else {
                $deferred.resolve(result.token);
            }
        }).fail(function() {
            $deferred.reject();
        });
        return $deferred.promise();
    }
}
