/**
 * TODO: Temporary variable that is used for check styles initialization.
 * TODO: Remove when all active projects will be rebuild or updated.
 *
 * @type {boolean}
 */
var isStylesInit = false;

/**
 * Perform one-time app initialisation on startup.
 *
 * @return {promise}   Promise of inited app.
 */
function initApp() {
    var appInit = new $.Deferred();
    initStyle(mainConfig.theme);
    if (ExternalLinkLoader.isAvailable()) {
        ExternalLinkLoader.initView().done(function() {
            appInit.resolve();
        }).fail(function() {
            appInit.reject();
        });
    } else {
        appInit.resolve();
    }
    return appInit.promise();
}

/**
 * Инициализирует главную страницу приложения.
 */
function initMainPage() {
    // Используется для сокрытия символа корзины на главной странице
    // т.к. в данном случае стандартное решение, исп. для других страниц
    // template7Data не работает
    var isShowAppName = (mainConfig.theme.header_setting_id == 'text') ||
        (mainConfig.theme.header_setting_id == 'icon_text');
    $('.js-app-name').text(isShowAppName ? mainConfig.app_name : '');
    if (!shopInstance || !mainConfig.showCart) {
        $('.js-cart').remove();
    }
    if (!isStylesInit) {
        initStyle(mainConfig.theme);
    }
    $(function() {
        renderButtons($('.main-menu-items').last(), mainConfig.instances);
    });
}

/**
 * Цепляет обработчик кнопки назад.
 *
 * @param {object} page - данные о странице f7;
 */
function initBackEvent(page) {
    $(document).off('backbutton');
    if(page.name !== 'index') {
        $(document).on('backbutton', onBackKeyDown);
    }
}

/**
 * Обработчик события кнопки "Назад"
 */
function onBackKeyDown() {
    mainView.router.load({url: 'index.html'});
}

/**
 * Initializes application style, defining on "Design" page.
 * Injects CSS onto existed CSS classes.
 *
 * @param {Object} theme                     Application theme.
 *
 * @param {string} theme.header_setting_id   Show only title, title and icon or
 *                                           only icon.
 *
 * @param {string} theme.sidebar_text                   Sidebar text color.
 * @param {string} theme.sidebar_background             Sidebar background
 *                                                      color.
 * @param {string} theme.header_text                    Header text color.
 * @param {string} theme.header_background              Header background color.
 * @param {string} theme.btn_background                 Main page buttons
 *                                                      background color.
 * @param {string} theme.interface_text                 Main page buttons
 *                                                      text color.
 * @param {string} theme.top_android_block              Main page background
 *                                                      color.
 * @param {string} theme.components_background          Components background
 *                                                      color.
 * @param {null|string} theme.sidebar_background_image  Sidebar background
 *                                                      image.
 * @param {string} theme.sidebar_delimeters             Sidebar delimeters
 *                                                      color.
 * @param {string} theme.sandwich_icon                  Sandwich icon color.
 */
function initStyle(theme) {
    isStylesInit = true;
    var $mainIcon = $('.js-main-icon');
    if ((theme.header_setting_id == 'icon') ||
        (theme.header_setting_id == 'icon_text')
    ) {
        $mainIcon
            .attr({src: mainConfig.icon})
            .css('display', 'block');
    } else {
        $mainIcon.remove();
    }

    var sheet = document.styleSheets[0];
    [
        '.panel-left {color: ' + theme.sidebar_text + ' !important;}',
        '.navbar {color: ' + theme.header_text + ' !important;}',
        '.navbar {background-color: ' + theme.header_background + ' !important;}',
        '.page .page-content {background-color: ' + theme.components_background + ' !important;}',
        '.page[data-page="index"] .page-content {background-color: ' + theme.top_android_block + ' !important;}',
        '.list-block .item-inner.special-item:after {background-color: ' + theme.sidebar_delimeters + ' !important;}',
        '.list-block ul.special-after:after {background-color: ' + theme.sidebar_delimeters + ' !important;}',
        '.list-block ul.special-before:before {background-color: ' + theme.sidebar_delimeters + ' !important;}',
        '.main-page-btn {background-color: ' + theme.btn_background + ';}',
        '.main-page-btn {color: ' + theme.interface_text + ';}',
        '.navbar-inner .fa-bars {color: ' + theme.sandwich_icon + ';}',
        '.navbar-inner .fa-bars {color: ' + theme.sandwich_icon + ';}',
    ].forEach(function(rule) {
        sheet.insertRule(rule, 0);
    });

    // Картинка в боковом меню
    if (theme.sidebar_background_image) {
        var $pl = $('.panel-left');
        var $lb = $('.list-block');
        $pl.css(
            'background-image', 'url(' + theme.sidebar_background_image + ')'
        );
        $pl.css('background-repeat', 'no-repeat');
        $pl.css('background-position', 'center top');
        $lb.addClass('background-transparent');

        var img = new Image();
        img.onload = function() {
            // 76x130 is default background image size.
            // Need to cover for backward compatibility.
            if ((img.width === 76) && (img.height === 130)) {
                $pl.css('background-size', 'cover');
            }
        };
        img.src = theme.sidebar_background_image;

        $('ul').addClass('background-transparent');
    } else {
        [
            '.panel-left {background-color: ' + theme.sidebar_background + ' !important;}',
            '.list-block {background-color: ' + theme.sidebar_background + ' !important;}',
            'ul {background-color: ' + theme.sidebar_background + ' !important;}',
        ].forEach(function(rule) {
            sheet.insertRule(rule, 0);
        });
    }

    $('.list-block ul')
        .addClass('special-after')
        .addClass('special-before');
    $('.list-block .item-inner').addClass('special-item');
}

/**
 * TODO: think about build-in selector and instances
 * Отрисовывает боковое меню.
 *
 * @param {object} $selector   Селектор, куда отрисовывать.
 * @param {Array} instances    Массив инстансов.
 */
function renderSideMenu($selector, instances) {
    $selector.html('');
    var instance;
    var $menuElement = $($('#menu-element').text());
    $menuElement.find('a').prop('href', 'index.html');
    if (mainConfig.theme.home_icon_image === null) {
        $menuElement.find('.icon').prop('src', schema + '://' + serverUrl + '/img/icon_sets/' +
            mainConfig.icon_set + '/home.png');
    } else {
        $menuElement.find('.icon').prop('src', mainConfig.theme.home_icon_image);
    }
    if (mainConfig.theme.home_title === null) {
        $menuElement.find('.item-title').text(l10n.common[lang].main_page);
    } else {
        $menuElement.find('.item-title').text(mainConfig.theme.home_title);
    }
    $selector.append($menuElement);

    for (var i = 0; i < instances.length; i++) {
        instance = instances[i];
        if (instance.hide_in_side_menu) {
            continue;
        }
        if (instance.callback!='openSlider') {
            $menuElement = $($('#menu-element').text());
            if (instance.callback) {
                $menuElement.find('a').removeClass('close-panel');
                appendCallback($menuElement.find('a'), instance);
            } else if (instance.href) {
                $menuElement.find('a').prop('href', instance.href);
            }
            $menuElement.find('.item-media')
                .prop('id', 'mainMenu-d-item' + instance.id + '-icon');
            $menuElement.find('.icon')
                .prop('src', instance.icon);
            $menuElement.find('.item-title')
                .prop('id', 'mainMenu-d-item' + instance.id + '-text')
                .text(instance.text);
            $selector.append($menuElement);
        }
    }

    $selector.append(renderLoginItem());
    $selector.append(renderAboutProgramItem());


    /**
     * Renders login side menu element.
     *
     * @return {jQuery}   Side menu element.
     */
    function renderAboutProgramItem() {
        // TODO: add special renderer class like for "account" page to be able
        // TODO: to add elements without breaking changes.
        if (!l10n['about']) {
            return null;
        }
        var $menuElement = $($('#menu-element').text());
        $menuElement.prop('id', 'privacy-policy');
        $menuElement.find('.icon').prop(
            'src', mainConfig.theme.about_icon
                       ? mainConfig.theme.about_icon
                       : 'img/about.png'
        );
        $menuElement
            .find('.item-title')
            .text(l10n['about'][lang].about_program);
        var $link = $menuElement.find('a');
        $link.prop('href', 'policy.html');
        return $menuElement;
    }

    /**
     * Renders login side menu element.
     *
     * @return {jQuery}   Side menu element.
     */
    function renderLoginItem() {
        if ((typeof(account) === 'undefined')
            || (mainConfig.registration_type === LoginForm.REGISTRATION_DISABLED)
        ) {
            return '';
        }

        var $menuElement = $($('#menu-element').text());
        $menuElement.prop('id', 'personal-cabinet');
        $menuElement.find('.icon')
                    .replaceWith('<i class="fa fa-2x fa-sign-in"></i>');

        var $link = $menuElement.find('a');
        $link.prop('href', 'login.html');

        framework7AppReady.then(function() {
            account.$inited.then(function() {
                $menuElement.find('.item-title').text(
                    account.isAuthorized()
                        ? l10n['auth_sidemenu'][lang].personal_cabinet
                        : l10n['auth_sidemenu'][lang].sign_in
                );
                account.isAuthorized()
                    && $menuElement.find('.item-title').append(
                        $('<span class="account-email"></span>')
                            .text(' ('+ account.options.email +')')
                       );
                $link.off('click').on('click', function(event) {
                    if (!account.isAuthorized()) {
                        mainView.router.load({url: 'login.html'});
                        return;
                    }
                    event.stopPropagation();
                    event.preventDefault();
                    accountOptions.renderSideMenu();
                });
            });
        });
        return $menuElement;
    }
}

/**
 * Применяет коллбэк к баннеру слайдера.
 *
 * @param {object} $a         Элемент для применения коллбэка.
 * @param {object} instance   Инстанс, содержащий коллбэки его параметры.
 */
function appendSliderCallback($a, instance) {
    $a.on('click', function() {
        var param = {};
        param.num = $(this).data('number');
        param.inf = instance.params;
        window[instance.callback].call(window, param);
    });
}

/**
 * Применяет коллбэк инстанса к элементу.
 *
 * @param {object} $a         Элемент для применения коллбэка.
 * @param {object} instance   Инстанс, содержащий коллбэки его параметры.
 */
function appendCallback($a, instance) {
    $a.on('click', function() {
        window[instance.callback].call(window, instance.params);
    });
}

/**
 * Отрисовывает кнопки главного меню приложения.
 *
 * @param {object} $selector   Селектор, куда отрисовывать.
 * @param {Array} instances    Массив инстансов.
 */
function renderButtons($selector, instances) {
    var i, bufData = [],
        linesCount = 0;
    for (i = 0; i < instances.length; i++) {
        if ((instances[i].hideButton === false)
            && (isInstanceInstalled(instances[i].id))) {
            bufData[linesCount] = instances[i];
            linesCount++;
        } else {
            if (instances[i].callback == 'openSlider' && instances[i].params != null) {
                // Отрисовываем слайдер
                var $sc = $('<div class="swiper-container">')
                              .append($('<div class="swiper-wrapper">'))
                              .append($('<div class="swiper-pagination">'));
                var $sw = $sc.find('.swiper-wrapper');
                var hasActiveSlides = false;
                for (var j = 0; j < instances[i].params.length; j++) {
                    if ((instances[i].params[j] != null)
                        && instances[i].params[j].img
                    ) {
                        var now = new Date();
                        var from = new Date(instances[i].params[j].from);
                        var to = new Date(instances[i].params[j].to);
                        if (instances[i].params[j].sliderLifetime == null
                            || instances[i].params[j].sliderLifetime == false
                            || (instances[i].params[j].sliderLifetime == true
                            && from <= now
                            && to >= now)) {
                            var bufstr = instances[i].params[j].img.substring(5);
                            bufstr = bufstr.substring(0, bufstr.length - 2);

                            var $slider = renderSlide(bufstr, j);
                            appendSliderCallback(
                                $slider.find('.image-container'), instances[i]
                            );
                            $sw.append($slider);
                            hasActiveSlides = true;
                        }
                    }
                }
                if (hasActiveSlides) {
                    $selector.append($sc);
                    var autoplay = false;
                    if (instances[i].autoplayDelay) {
                        autoplay = instances[i].autoplayDelay * 1000;
                    }
                    myApp.swiper($sc, {
                        pagination: $sc.find('.swiper-pagination'),
                        autoplay: autoplay,
                    });
                }
            }
        }
    }
    linesCount = Math.ceil(linesCount / 2);
    for (i = 0; i < linesCount; i++) {
        var $line = $('<div class="line">');
        $line.append(createButton(bufData[i * 2]));
        $line.append(createButton(bufData[i * 2 + 1]));
        $selector.append($line);
    }


    /**
     * Renders one slider from given image source.
     *
     * @param {string} source   Slider image source.
     * @param {number} number   Slider number.
     *
     * @return {jQuery}   Slider jQuery object.
     */
    function renderSlide(source, number) {
        return $('<div class="swiper-slide">').append(
            $('<div class="image-container">')
                .data('number', number)
                .css({backgroundImage: 'url("' + source + '")'})
        );
    }
}

/**
 * "Отрисовка" кнопки меню.
 *
 * @param {object} data - jQuery Объект с настройками кнопки.
 *
 * @return {object}  jQuery Объект кнопки.
 */
function createButton(data) {
    if (data === null || data === undefined) {
        return '';
    }
    var $el = $('<a>').addClass('btn main-page-btn');
    if (!(data.buttonBackground === '#ffffff'
            || data.buttonBackground === 'rgb(255, 255, 255)'
            || data.buttonBackground === undefined)
    ) {
        $el.css('background-color', data.buttonBackground);
    }
    $el.append(
        $('<div>')
            .append(
                $('<div>').append($('<img>').prop('src', data.icon))
            )
            .append($('<span>').text(data.text))
    );

    if (!data.callback) {
        $el.prop('href', data.href);
        return $el;
    }

    switch (data.callback) {
    case 'openStore':
        $el.prop('href', data.href);
        break;
    case 'openLink':
    default:
        $el.on('click', function() {
            window[data.callback].call(window, data.params);
        });
    }
    return $el;
}

/**
 * Handler required for push notifications.
 */
document.addEventListener('deviceready', function() {
    var oneSignal = window.plugins.OneSignal;
    oneSignal
        .startInit(onesignal_app_id)
        .handleNotificationOpened(notificationOpenedCallback)
        .inFocusDisplaying(window.plugins.OneSignal.OSInFocusDisplayOption.None)
        .endInit();
    oneSignal.sendTag('app_id', pushTag);


    /**
     * Executes some action on notification.
     *
     * @param {object} data - JSON data.
     */
    function notificationOpenedCallback(data) {
        var additionalData = data.notification.payload.additionalData;
        data = (typeof additionalData === 'string')
                    ? JSON.parse(additionalData)
                    : additionalData;
        switch (data.action) {
        case 'call':
            document.location = 'tel:' + data.phone;
            break;
        case 'open_plugin':
            framework7AppReady.then(function() {
                account.$inited.then(function() {
                    mainView.router.load({url: data.plugin_id + '.html'});
                });
            });
            break;
        case 'open_news':
            framework7AppReady.then(function() {
                mainView.router.load({
                    url: data.instance_id + '.html?new=' + data.new_id,
                });
            });
            break;
        case 'open_category':
            framework7AppReady.then(function() {
                mainView.router.load({
                    url: data.instance_id + '.html' +
                    '?category_id=' + data.category_id,
                });
            });
            break;
        case 'open_product':
            framework7AppReady.then(function() {
                mainView.router.load({
                    url: data.instance_id + '.html' +
                    '?productId=' + data.product_id,
                });
            });
        }
    }
}, false);

/**
 * Handles URL opened from custom scheme.
 *
 * @param {string} url   Url that was invoked.
 */
function handleOpenURL(url) {
    var parts = url.split('/');
    if (parts < 2) {
        return;
    }
    var command = parts[2];
    switch (command) {
    case 'product':
        var instance_id = parts[3];
        var product_id = parts[4];
        framework7AppFullyInited.then(function() {
            mainView.router.load({
                url: instance_id + '.html?productId=' + product_id,
            });
        });
        break;
    }
}

/**
 * Binds event which displays icon on each page loading.
 */
function bindDisplayNavbarIconEvent() {
    myApp.onPageInit('*', displayNavbarIcon);
}

/**
 * Displays header icon.
 */
function displayNavbarIcon() {
    $('.js-main-icon').css({display: 'block'})
                      .prop('src', mainConfig.icon);
}

// Init geolocation logic
framework7AppReady.then(function() {
    $('.js-set-city').click(function(event) {
        event.preventDefault();
        var city = $('.city-choose-popup [name="city"]').val();
        if (city === '') {
            alert(l10n.common[lang].enter_city);
            return false;
        }

        var $btn = $(this);
        var initText = $btn.text();
        $btn.text(l10n.common[lang].checking);
        Geocoder.checkCityExists(city).done(function(resolved) {
            geocoder.setCity(resolved);
            renewCityTitle();
            myApp.closeModal('.city-choose-popup', true);
        }).fail(function() {
            alert(l10n.common[lang].city_not_found);
        }).always(function() {
            $btn.text(initText);
        });

        return false;
    });
});

/**
 * Renews city title with current geocode detected city.
 */
function renewCityTitle() {
    $('.js-city').text(geocoder.getCityName());
    $('.js-current-city').text(geocoder.getCityName());
}

/**
 * Tries to detect user city.
 */
framework7AppReady.then(function() {
    $('.js-re-geolocate').click(function(event) {
        myApp.showPreloader(l10n.common[lang].try_find_you);
        event.preventDefault();
        event.stopPropagation();
        geocoder.reGeolocate().done(function(city) {
            geocoder.setCity(city);
            renewCityTitle(city);
        }).fail(function() {
            myApp.alert(l10n.common[lang].cant_find);
        }).always(function() {
            myApp.hidePreloader();
        });
    });
});
