// Содержит вспомогательные скрипты:
// 1. Не относящиеся к Framework 7
// 2. Статичные (не меняющиеся во время компиляции)
// 3. Не генерирующие напрямую страницы и их элементы (например, главную).

// Support for old Android versions (4+)
if (typeof Object.assign !== 'function') {
    Object.assign = function(target, varArgs) {
        'use strict';
        if (target == null) {
            throw new TypeError('Cannot convert undefined or null to object');
        }
        var to = Object(target);
        for (var index = 1; index < arguments.length; index++) {
            var nextSource = arguments[index];

            if (nextSource != null) {
                for (var nextKey in nextSource) {
                    if (Object.prototype
                            .hasOwnProperty.call(nextSource, nextKey)) {
                        to[nextKey] = nextSource[nextKey];
                    }
                }
            }
        }
        return to;
    };
}

// Добавляет C# версию форматирования строки.
if (!String.prototype.format) {
    String.prototype.format = function() {
        var args = arguments;
        return this.replace(/{(\d+)}/g, function(match, number) {
            return typeof args[number] != 'undefined'
                ? args[number]
                : match;
        });
    };
}

/**
 * Возвращает значение GET-параметра из строки URL.
 *
 * @param {string} val  Имя параметра для получения значения.
 *
 * @return {string}   Значение GET параметра.
 */
function parseQuery(val) {
    var result = '',
        tmp = [];
    location.search.substr(1).split('&').forEach(function(item) {
        tmp = item.split('=');
        if (tmp[0] === val) result = decodeURIComponent(tmp[1]);
    });
    return result;
}

/**
 * Показывает, запущено приложение в конструкторе
 * или на реальном устройстве.
 *
 * @return {boolean}   Да или нет.
 */
function isProduction() {
    return (location.hostname.indexOf(serverUrl) == -1);
}

/**
 * Возвращает конфиг инстанса.
 *
 * @param {number} instanceID  ID инстанса, для которого надо получить конфиг
 *
 * @return {string}   Конфиг в формате JSON.
 */
function getConfig(instanceID) {
    var currentVersion = '1.0.0';
    if (typeof version !== 'undefined') {
        currentVersion = version;
    }

    return $.get(schema + '://' + serverUrl + '/site/get-config', {
        instance_id: instanceID,
        version: currentVersion,
    });
}

/**
 * Если запущено на телефоне, обновляет счетчик запусков.
 */
function updateLaunchCounter() {
    if (isProduction()) {
        $.get(schema + '://' + serverUrl + '/app/update-counter', {
            app_id: appID,
            counter: 'runs'
        });
    }
}

/**
 * Показывает, запущено ли приложение в первый раз (для статистики).
 *
 * @return {boolean}  Да или нет.
 */
function isFirstLaunch() {
    if (!isProduction) {
        return false;
    }
    var result = localStorage.getItem('firstLaunch');
    if (result == null) {
        localStorage['firstLaunch'] = true;
        return true;
    }
    return false;
}

/**
 * Если запущено на телефоне, обновляет счетчик запусков.
 */
function updateInstallationCounter() {
    if (isFirstLaunch()) {
        $.get(schema + '://' + serverUrl + '/app/update-counter', {
            app_id: appID,
            counter: 'installs'
        });
    }
}

/**
 * Проверяет, установлен ли инстанс с заданным ID в приложении.
 * Это необходимо для того, чтобы дать возможность добавлять компоненты в
 * админке, и при этом не сломать уже установленные приложения.
 *
 * @param {number} instanceID   ID инстанса для проверки.
 *
 * @return {boolean}   Да или нет.
 */
function isInstanceInstalled(instanceID) {
    return (installedInstances.indexOf(instanceID) != -1);
}

/**
 * Генерирует из массива инстансов массив шаблонов для F7.
 *
 * @param {Array} instances   Шаблоны страниц.
 *
 * @return {Array}  Конфиг шаблонов страниц.
 */
function initF7Templates(instances) {
    var ibv = false;
    var templates = [];
    if (shopInstance && mainConfig.showCart) {
        ibv = true;
    }
    for (var i = 0; i < instances.length; i++) {
        templates['url:' + instances[i].href] = {
            Num: instances[i].id,
            Title: instances[i].text,
            IsBasketVisible: ibv
        };
    }
    return templates;
}

/**
 * Возвращает первый встреченный инстанс магазина из списка или null,
 * если таковых нет.
 *
 * @param {object} instances   Набор инстансов
 *
 * @return {null|object}   Объект инстанса или null.
 */
function detectShopInstance(instances) {
    for (var i = 0; i < instances.length; i++) {
        if (instances[i].isShop) {
            return instances[i];
        }
    }
    return null;
}

/**
 * Returns platform on which application is running.
 *
 * @return {string}   Platform name.
 */
function getPlatform() {
    return 'android';
}

/**
 * Forces redraw of given HTML element.
 * Used primarily for iOS to enforces redraw after updating element.
 *
 * @param {object} element   Element to be redrawed.
 */
function forceRedraw(element) {
    if (!element) {
        return;
    }
    var n = document.createTextNode(' ');
    var disp = element.style.display;
    element.appendChild(n);
    element.style.display = 'none';
    setTimeout(function() {
        element.style.display = disp;
        n.parentNode.removeChild(n);
    }, 100);
}

/**
 * Функция, выполняющаяся при нажатии на кнопку компонента web-ссылка
 *
 * @param {string} params   URL страницы, которую необходимо открыть
 */
function openLink(params) {
    window.open(params);
}


/** @var {object} wizViewManager */

/**
 * Wrapper for cordova external webview plugin.
 *
 * @param {Object} options   Options for plugin initialization.
 */
function ExternalLinkLoader(options) {
    options = Object.assign({
        link: null,
        container: null,
        onInit: function() {},
        onInitLoaded: function() {},
        onError: function() {},
        onShow: function() {},
        onHide: function() {},
    }, options);

    var STATE_OPENED = 1;
    var STATE_HIDE = 0;
    var currentState = STATE_OPENED;
    var self = this;
    var objectL10n = {
        'ru-RU': {
            'disclaimer': 'Данный компонент не может быть просмотрен в эмуляторе.<br>' +
                          'Тем не менее, он будет корректно работать на телефоне.'
        },
        'en-US': {
            'disclaimer': 'This component has no preview in emulator.<br>' +
                          'However, it will work on phones.'
        }
    };


    /**
     * Hides link.
     */
    this.hideLink = function() {
        if (ExternalLinkLoader.isAvailable()) {
            currentState = STATE_HIDE;
            wizViewManager.hide(
                ExternalLinkLoader.VIEW_NAME, {},
                function() {}, options.onError
            );
            options.onHide();  // On iOS successful callback for this func
                               // does not work.
        }
    };

    /**
     * Shows link.
     */
    this.showLink = function() {
        if (ExternalLinkLoader.isAvailable()) {
            currentState = STATE_OPENED;
            wizViewManager.show(
                ExternalLinkLoader.VIEW_NAME, {},
                function() {}, options.onError
            );
            options.onShow();  // For unification with hideLink method.
        }
    };

    if (!ExternalLinkLoader.isAvailable()) {
        $(options.container).append(
            $('<div class="page-content">')
                .css({
                    padding: '70px 20px 0 20px',
                    textAlign: 'center',
                })
                .html(objectL10n[lang].disclaimer)
        );
        return;
    }

    options.onInit();
    wizViewManager.load(ExternalLinkLoader.VIEW_NAME, options.link, function() {
        options.onInitLoaded(self);
        ExternalLinkLoader.setLayout();
        if (currentState === STATE_OPENED) {
            self.showLink();
        }
    }, options.onError);
}

/**
 * External view name.
 *
 * @type {string}
 */
ExternalLinkLoader.VIEW_NAME = 'in-app-site';

/**
 * Checks whether external webview plugin is available.
 *
 * @return {boolean}   Yes or no.
 */
ExternalLinkLoader.isAvailable = function() {
    return typeof wizViewManager !== 'undefined';
};

/**
 * Set external view layout to fit whole screen with padding for menu.
 */
ExternalLinkLoader.setLayout = function() {
    var NAVBAR_HEIGHT = 112;  // Twice as much as navbar
    wizViewManager.setLayout(ExternalLinkLoader.VIEW_NAME, {
        top: NAVBAR_HEIGHT,
        height: (window.innerHeight * 2) - NAVBAR_HEIGHT
    });
};

/**
 * Init external URL view.
 *
 * @return {promise}   Promise of view being initialized.
 */
ExternalLinkLoader.initView = function() {
    var init = new $.Deferred();
    wizViewManager.create(ExternalLinkLoader.VIEW_NAME, {
        src: schema + '://' + serverUrl + '/blank.txt',
    }, function() {
        wizViewManager.hide(ExternalLinkLoader.VIEW_NAME);
        window.addEventListener('orientationchange', function() {
            // Timeout required because redrawing get some time.
            setTimeout(function() {
                ExternalLinkLoader.setLayout();
            }, 200);
        });
        init.resolve();
    }, function() {
        init.reject();
    });
    setTimeout(function() {
        init.reject();
    }, 5000);
    return init.promise();
};


/**
 * GeoCoder Wrapper API.
 */
var Geocoder = function() {
    var self = this;
    this.city = null;
    this.inited = new $.Deferred();

    /**
     * Initializes geocoder functions.
     */
    this.init = function() {
        this.city = localStorage.getItem('city');
        if (this.city && this.city !== 'undefined') {
            self.inited.resolve();
            return;
        }

        this.reGeolocate().done(function(city) {
            self.setCity(city);
        }).always(function() {
            self.inited.resolve();
        });
    };

    this.reGeolocate = function() {
        var $deferred = new $.Deferred();

        Geocoder.getCoords().done(function(coords) {
            Geocoder.getCityByCoords(coords).done(function(city) {
                $deferred.resolve(city);
            }).fail(function() {
                $deferred.reject();
            });
        }).fail(function() {
            $deferred.reject();
        });
        return $deferred.promise();
    };

    /**
     * Saves city to the object and permanent storage.
     *
     * @param {string} city   City to set.
     */
    this.setCity = function(city) {
        localStorage['city'] = city;
        self.city = city;
    };

    /**
     * Returns city name.
     *
     * @return {string}   City name.
     */
    this.getCityName = function() {
        return this.city || l10n.common[lang].another_city;
    };
};

/**
 * Returns device coordinates.
 *
 * @return {Promise}  Promise with coordinates as array [latitude, longitude].
 */
Geocoder.getCoords = function() {
    var deferred = new $.Deferred();
    if (!navigator.geolocation) {
        deferred.reject();
        return deferred.promise();
    }

    navigator.geolocation.getCurrentPosition(function(position) {
        deferred.resolve([
            position.coords.latitude,
            position.coords.longitude
        ]);
    }, function() {
        deferred.reject();
    }, {
        timeout: 5000,
        maximumAge: 30000,
    });
    return deferred.promise();
};

/**
 * Returns city name by coordinates.
 *
 * @param {Array} coords   Array where first key is latitude,
 *                         second - longitude.
 *
 * @return {Promise}  Promise with city.
 */
Geocoder.getCityByCoords = function(coords) {
    var deferred = new $.Deferred();

    $.get('https://maps.googleapis.com/maps/api/geocode/json?' + [
            'latlng=' + coords.join(','),
            'sensor=true',
            'key=' + gmapsKey,
            'language=' + (lang === 'ru-RU' ? 'ru' : 'en'),
        ].join('&')
    ).done(function(results) {
        try {
            for (var i = 0; i < results.results.length; i++) {
                var result = results.results[i];
                for (var j = 0; j < result.address_components.length; j++) {
                    var component = result.address_components[j];
                    if (component.types.indexOf('locality') !== -1) {
                        deferred.resolve(component.short_name);
                        return;
                    }
                }
            }
        } catch (e) {
            // Does nothing, auto reject
        }
        deferred.reject();
    }).fail(function() {
        deferred.reject();
    });

    return deferred.promise();
};

/**
 * Check given city for existance.
 *
 * @param {string} city   City to be checked.
 *
 * @return {Promise}  Promise with resolved city name or reject.
 */
Geocoder.checkCityExists = function(city) {
    var deferred = new $.Deferred();

    $.get('https://maps.googleapis.com/maps/api/geocode/json?' + [
        'address=' + city,
        'key=' + gmapsKey,
        'language=' + (lang === 'ru-RU' ? 'ru' : 'en'),
    ].join('&')
    ).done(function(results) {
        var cityUpperCase = city.toUpperCase();
        try {
            for (var i = 0; i < results.results.length; i++) {
                var result = results.results[i];
                for (var j = 0; j < result.address_components.length; j++) {
                    var component = result.address_components[j];
                    if ((component.types.indexOf('locality') !== -1)
                            && ((component.short_name.toUpperCase()
                                    === cityUpperCase)
                                || (component.long_name.toUpperCase()
                                        === cityUpperCase))
                    ) {
                        deferred.resolve(component.short_name);
                        return;
                    }
                }
            }
        } catch (e) {
            // Does nothing, auto reject
        }
        deferred.reject();
    }).fail(function() {
        deferred.reject();
    });

    return deferred.promise();
};

/**
 * Copies given string to clipboard.
 *
 * @param {string} str   String to be copied.
 */
function copyStringToClipboard(str) {
    if (cordova && cordova.plugins && cordova.plugins.clipboard) {
        cordova.plugins.clipboard.copy(str);
    } else {
        myApp.alert('Плагин не установлен');
    }
}

/**
 * Class for work with semantic versions.
 */
var SemVer = function() {};
SemVer.isLowerThan = function(version1, version2) {
    var parts1 = version1.split('.');
    var parts2 = version2.split('.');

    if (parts1[0] !== parts2[0]) {
        return parts1[0] < parts2[0];
    }
    if (parts1[1] !== parts2[1]) {
        return parts1[1] < parts2[1];
    }
    return parts1[2] < parts2[2];
};


/**
 * Shows soft update window.
 */
function showSoftUpdateWindow() {
    var buttons = {};
    if (mainConfig.google_play_link) {
        buttons.push({
            text: l10n.common[lang].update,
            onClick: function() {
                window.open(mainConfig.google_play_link);
            },
            close: false,
        });
    }
    buttons.push({text: 'OK'});
    myApp.modal({
        text: l10n.common[lang].update_notification,
        buttons: buttons,
    });
}

/**
 * Shows hard update window.
 */
function showHardUpdateWindow() {
    myApp.modal({
        text: l10n.common[lang].update_required,
        buttons: mainConfig.google_play_link
            ? [{
                  text: l10n.common[lang].update,
                  onClick: function() {
                      window.open(mainConfig.google_play_link);
                  },
                  close: false,
              }]
            : [],
    });
}
